`timescale 1ns / 1ps

module cfg_2to1
(
    cfg_clk_i,
    cfg_rst_i,
    
    cfg0_sel_i,
    cfg0_wr_i,
    cfg0_ena_i,
    cfg0_addr_i,///byte address
    cfg0_wdata_i,
    cfg0_strb_i,
    cfg0_rdy_o,
    cfg0_rdata_o,
    cfg0_slverr_o,
    
    cfg1_sel_i,
    cfg1_wr_i,
    cfg1_ena_i,
    cfg1_addr_i,///byte address
    cfg1_wdata_i,
    cfg1_strb_i,
    cfg1_rdy_o,
    cfg1_rdata_o,
    cfg1_slverr_o,
    
    cfg_sel_o,
    cfg_wr_o,
    cfg_ena_o,
    cfg_addr_o,///byte address
    cfg_wdata_o,
    cfg_strb_o,
    cfg_rdy_i,
    cfg_rdata_i,
    cfg_slverr_i
    
);
                            
input                                   cfg_clk_i;
input                                   cfg_rst_i;

input                                   cfg0_sel_i/* synthesis syn_keep=1 */;
input                                   cfg0_wr_i/* synthesis syn_keep=1 */;
input                                   cfg0_ena_i/* synthesis syn_keep=1 */;
input [21:0]                            cfg0_addr_i;///byte address
input [31:0]                            cfg0_wdata_i/* synthesis syn_keep=1 */;
input [3:0]                             cfg0_strb_i/* synthesis syn_keep=1 */;
output reg                              cfg0_rdy_o/* synthesis preserve = 1 */;
output reg[31:0]                        cfg0_rdata_o/* synthesis preserve = 1 */;
output reg                              cfg0_slverr_o/* synthesis preserve = 1 */;
       
input                                   cfg1_sel_i/* synthesis syn_keep=1 */;
input                                   cfg1_wr_i/* synthesis syn_keep=1 */;
input                                   cfg1_ena_i/* synthesis syn_keep=1 */;
input [21:0]                            cfg1_addr_i;///byte address
input [31:0]                            cfg1_wdata_i/* synthesis syn_keep=1 */;
input [3:0]                             cfg1_strb_i/* synthesis syn_keep=1 */;
output reg                              cfg1_rdy_o/* synthesis preserve = 1 */;
output reg[31:0]                        cfg1_rdata_o/* synthesis preserve = 1 */;
output reg                              cfg1_slverr_o/* synthesis preserve = 1 */;

output reg                              cfg_sel_o/* synthesis preserve = 1 */;
output reg                              cfg_wr_o/* synthesis preserve = 1 */;
output reg                              cfg_ena_o/* synthesis preserve = 1 */;
output reg[21:0]                        cfg_addr_o/* synthesis preserve = 1 */;///byte address
output reg[31:0]                        cfg_wdata_o/* synthesis preserve = 1 */;
output reg[3:0]                         cfg_strb_o/* synthesis preserve = 1 */;
input                                   cfg_rdy_i/* synthesis syn_keep=1 */;
input [31:0]                            cfg_rdata_i/* synthesis syn_keep=1 */;
input                                   cfg_slverr_i/* synthesis syn_keep=1 */;

localparam                              S_IDLE    = 2'b00;
localparam                              S_BUSY    = 2'b01;
localparam                              S_DONE    = 2'b10;

reg [1:0]                               state = S_IDLE/* synthesis preserve = 1 */;
reg [1:0]                               next_state/* synthesis preserve = 1 */;     
reg [1:0]                               cfg_bus_sel = 0/* synthesis preserve = 1 */;  


always@(posedge cfg_clk_i) begin
    if((cfg_rdy_i == 1'b1) || (state == S_DONE)) begin
        cfg_sel_o    <= 1'b0;
        cfg_wr_o     <= 1'b0;
        cfg_ena_o    <= 1'b0;
    end
    else if(cfg_bus_sel[0] == 1'b1) begin
        cfg_sel_o    <= cfg0_sel_i;
        cfg_wr_o     <= cfg0_wr_i;
        cfg_ena_o    <= cfg0_ena_i;
    end
    else if(cfg_bus_sel[1] == 1'b1) begin
        cfg_sel_o    <= cfg1_sel_i;
        cfg_wr_o     <= cfg1_wr_i;
        cfg_ena_o    <= cfg1_ena_i;
    end
    else begin
        cfg_sel_o    <= 1'b0;
        cfg_wr_o     <= 1'b0;
        cfg_ena_o    <= 1'b0;
    end
end

always@(posedge cfg_clk_i) begin
    if(cfg_bus_sel[0] == 1'b1) begin
        cfg_addr_o   <= cfg0_addr_i;
        cfg_wdata_o  <= cfg0_wdata_i;
        cfg_strb_o   <= cfg0_strb_i;
        cfg0_rdy_o   <= cfg_rdy_i;
        cfg0_slverr_o<= cfg_slverr_i;
        cfg1_rdy_o   <= 1'b0;
        cfg1_slverr_o<= 1'b0;
    end
    else if(cfg_bus_sel[1] == 1'b1) begin
        cfg_addr_o   <= cfg1_addr_i;
        cfg_wdata_o  <= cfg1_wdata_i;
        cfg_strb_o   <= cfg1_strb_i;
        cfg0_rdy_o   <= 1'b0;
        cfg0_slverr_o<= 1'b0;
        cfg1_rdy_o   <= cfg_rdy_i;
        cfg1_slverr_o<= cfg_slverr_i;
    end
    else begin
        cfg_addr_o   <= 24'b0;  
        cfg_wdata_o  <= 32'b0;
        cfg_strb_o   <= 4'b0;
        cfg0_rdy_o   <= 1'b0;
        cfg0_slverr_o<= 1'b0;
        cfg1_rdy_o   <= 1'b0;
        cfg1_slverr_o<= 1'b0;
    end
end

always@(posedge cfg_clk_i) begin
    cfg0_rdata_o <= cfg_rdata_i;
    cfg1_rdata_o <= cfg_rdata_i;
end

always@(posedge cfg_clk_i) begin
    if(cfg_rst_i)
        state <= S_IDLE;
    else 
        state <= next_state;
end

always@(*) begin
    case(state)
        S_IDLE : 
            begin
                if(cfg0_sel_i | cfg1_sel_i)
                    next_state = S_BUSY;
                else 
                    next_state = S_IDLE;
            end
        
        S_BUSY :
            begin
                if((cfg_sel_o == 1'b1) & (cfg_ena_o == 1'b1) & (cfg_rdy_i == 1'b1)) 
                    next_state = S_DONE;
                else 
                    next_state = S_BUSY;
            end
        S_DONE :
            begin
                next_state = S_IDLE;
            end
        
        default :
            begin
                next_state = S_IDLE;
            end
    endcase
end

always@(posedge cfg_clk_i) begin
    if((state == S_IDLE) && (cfg0_sel_i == 1'b1))
        cfg_bus_sel  <= 2'b01;
    else if((state == S_IDLE) && (cfg1_sel_i == 1'b1))
        cfg_bus_sel  <= 2'b10;
    else if((state == S_IDLE) || (state == S_DONE))
        cfg_bus_sel  <= 2'b00;
end


endmodule
