//**********************************************************************
//  Project: 
//  File:  
//  Description:      GW伪双端口RAM RTL
//  Author:           liaobin
//  Target Devices:   
//  Timestamp:        2020/12/14
//---------------------------------------------------------------------- 

module gowin_sdpram #(
  parameter RAM_WIDTH  = 18,                      // Specify RAM data width
  parameter RAM_DEPTH  = 128,                     // Specify RAM depth (number of entries)
  parameter RAM_PERFORMANCE = "bypass",           // Select "pileline" or "bypass"  
  parameter RAM_STYLE = "block_ram"               // "block_ram"  "distributed_ram"  "registers"
)
(
  input [$clog2(RAM_DEPTH-1)-1:0] i_addra,        // address bus, width determined from RAM_DEPTH
  input [$clog2(RAM_DEPTH-1)-1:0] i_addrb,        // address bus, width determined from RAM_DEPTH
  input [RAM_WIDTH-1:0]           i_dina,         // RAM input data
  input                           i_clka,         // Write clock
  input                           i_clkb,         // Read clock
  input                           i_wea,          // Write enable
  input                           i_enb,          // Read enable
  input                           i_rstb,         // reset register(pipeline mode) 注：在写操作时候，rst必须为0 
  input                           i_regceb,       // Output register enable
  output [RAM_WIDTH-1:0]          o_doutb         // RAM output data
);


localparam           ADDR_WIDTH = $clog2(RAM_DEPTH-1); 

//reg [RAM_WIDTH-1:0]  mem_data[0:2**ADDR_WIDTH-1]/* synthesis syn_ramstyle = "distributed_ram" */;
reg [RAM_WIDTH-1:0]  ram_data; 


generate
    if(RAM_STYLE == "block_ram") begin
    
        reg [RAM_WIDTH-1:0]  mem_data[0:2**ADDR_WIDTH-1]/* synthesis syn_ramstyle = "block_ram" */;
        

        always@(posedge i_clka) begin
            if(i_wea)
                mem_data[i_addra] <= i_dina;
        end
        
        always@(posedge i_clkb) begin
            if(i_enb)
                ram_data <= mem_data[i_addrb];
        end

    
    end
    else if(RAM_STYLE == "distributed_ram") begin
    
        reg [RAM_WIDTH-1:0]  mem_data[0:2**ADDR_WIDTH-1]/* synthesis syn_ramstyle = "distributed_ram" */;
        
        always@(posedge i_clka) begin
            if(i_wea)
                mem_data[i_addra] <= i_dina;
        end
        
        always@(posedge i_clkb) begin
            if(i_enb)
                ram_data <= mem_data[i_addrb];
        end
    
    end
    else begin
    
        reg [RAM_WIDTH-1:0]  mem_data[0:2**ADDR_WIDTH-1]/* synthesis syn_ramstyle = "registers" */;

        always@(posedge i_clka) begin
            if(i_wea)
                mem_data[i_addra] <= i_dina;
        end
        
        always@(posedge i_clkb) begin
            if(i_enb)
                ram_data <= mem_data[i_addrb];
        end
        
    end


endgenerate






generate
    if(RAM_PERFORMANCE == "pileline") begin : output_register
        
        reg [RAM_WIDTH-1:0]   dout_reg={RAM_WIDTH{1'b0}};
        
        always@(posedge i_clkb) begin
            if(i_rstb)  
                dout_reg <= 0;
            else if(i_regceb)
                dout_reg <= ram_data;
        end
        
        assign  o_doutb = dout_reg;
        
    end
    else begin : no_output_register
    
        assign  o_doutb = ram_data;
    
    end
           
endgenerate



endmodule 



