
`timescale 1ps/1ps

module gowin_srio_maints (
    input             clk_i,
    input             rst_i,
    
    output  reg       cfg_sel_o/* synthesis preserve = 1 */,
    output  reg       cfg_wr_o/* synthesis preserve = 1 */,
    output  reg       cfg_ena_o/* synthesis preserve = 1 */,
    output [21:0]     cfg_addr_o,
    output [31:0]     cfg_wdata_o,
    input             cfg_rdy_i,
    input [31:0]      cfg_rdata_i,
    
    input             srio_trac_start_i,
    output reg        maint_done_o/* synthesis preserve = 1 */,
    output reg        maint_autocheck_error_o/* synthesis preserve = 1 */
 );

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
///parameter
    localparam [8:0] STARTING_ADDRESS       = 9'h0;
    localparam       NUMBER_OF_INSTRUCTIONS = 20;
    
    localparam       READ      = 1'b1;
    localparam       WRITE     = 1'b0;
    localparam       LOCAL     = 1'b0;
    localparam       REMTE     = 1'b1;
    
    localparam [3:0] MAINT     = 4'h8;
    
    localparam [3:0] RDREQ     = 4'h0;
    localparam [3:0] WRREQ     = 4'h1;
    
    localparam [7:0] HOP_LOCAL = 8'h00;
    localparam [7:0] HOP_REMTE = 8'hFF;
    
                                     // RSVD, LOCAL/REMTE, ADDRESS,    RD/WR, DATA,         DATA MASK
    localparam [63:0] MAINTENANCE0   = {2'b0, 3'b0,  REMTE,       21'h000000, READ,  32'h0270000E, 4'hF};
    localparam [63:0] MAINTENANCE1   = {2'b0, 3'b0,  LOCAL,       21'h000000, READ,  32'h0270000E, 4'hF};
///    localparam [63:0] MAINTENANCE2   = {2'b0, 3'b0,  REMTE,       21'h000060, WRITE, 32'hDEADBEEF, 4'h0};
///    localparam [63:0] MAINTENANCE3   = {2'b0, 3'b0,  REMTE,       21'h000060, READ,  32'h00ADBEEF, 4'hF};
    localparam [63:0] MAINTENANCE2   = {2'b0, 3'b0,  REMTE,       21'h000060, WRITE, 32'h04030102, 4'h0};
    localparam [63:0] MAINTENANCE3   = {2'b0, 3'b0,  REMTE,       21'h000060, READ,  32'h00030102, 4'hF};
                                                             
    localparam [63:0] MAINTENANCE4   = {2'b0, 3'b0,  LOCAL,       21'h00005C, WRITE, 32'hFFFFFFFF, 4'h0};
    localparam [63:0] MAINTENANCE5   = {2'b0, 3'b0,  LOCAL,       21'h00005C, READ,  32'h7FE00000, 4'hF};
    localparam [63:0] MAINTENANCE6   = {2'b0, 3'b0,  REMTE,       21'h00005C, WRITE, 32'hFFFFFFFF, 4'h0};
    localparam [63:0] MAINTENANCE7   = {2'b0, 3'b0,  REMTE,       21'h00005C, READ,  32'h7FE00000, 4'hF};
                                                             
    localparam [63:0] MAINTENANCE8   = {2'b0, 3'b0,  REMTE,       21'h010000, WRITE, 32'hFF020304, 4'h0};
    localparam [63:0] MAINTENANCE9   = {2'b0, 3'b0,  LOCAL,       21'h010000, READ,  32'h00020304, 4'h0};
    localparam [63:0] MAINTENANCE10  = {2'b0, 3'b0,  LOCAL,       21'h010000, WRITE, 32'h00030405, 4'h0};
    localparam [63:0] MAINTENANCE11  = {2'b0, 3'b0,  REMTE,       21'h010000, READ,  32'h00030405, 4'h0};
                                                          
    localparam [63:0] MAINTENANCE12  = {2'b0, 3'b0,  REMTE,       21'h000120, WRITE, 32'hFFFFFFFF, 4'h0};
    localparam [63:0] MAINTENANCE13  = {2'b0, 3'b0,  REMTE,       21'h000120, READ,  32'hFFFFFF00, 4'h0};
    localparam [63:0] MAINTENANCE14  = {2'b0, 3'b0,  LOCAL,       21'h000124, WRITE, 32'h01020304, 4'h0};
    localparam [63:0] MAINTENANCE15  = {2'b0, 3'b0,  LOCAL,       21'h000124, READ,  32'h01020300, 4'h0};
                                                               
    localparam [63:0] MAINTENANCE16  = {2'b0, 3'b0,  LOCAL,       21'h000410, READ,  32'h00007FC8, 4'h0};
    localparam [63:0] MAINTENANCE17  = {2'b0, 3'b0,  LOCAL,       21'h000430, READ,  32'h00107FC8, 4'h0};
    localparam [63:0] MAINTENANCE18  = {2'b0, 3'b0,  REMTE,       21'h000450, READ,  32'h00207FC8, 4'h0};
    localparam [63:0] MAINTENANCE19  = {2'b0, 3'b0,  REMTE,       21'h000470, READ,  32'h00307FC8, 4'h0};
    
    wire [63:0] instruction [0:511];
    
    localparam [64*NUMBER_OF_INSTRUCTIONS-1:0] maintenance_instruction = {
    MAINTENANCE19,MAINTENANCE18,MAINTENANCE17,MAINTENANCE16,MAINTENANCE15,MAINTENANCE14,MAINTENANCE13,
    MAINTENANCE12,MAINTENANCE11,MAINTENANCE10,MAINTENANCE9, MAINTENANCE8, MAINTENANCE7, MAINTENANCE6,
    MAINTENANCE5, MAINTENANCE4, MAINTENANCE3, MAINTENANCE2, MAINTENANCE1, MAINTENANCE0  }; 

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
///signal        
    wire [63:0] maint_data_out_d/* synthesis syn_keep=1 */;
    reg  [61:0] maint_data_out/* synthesis preserve = 1 */; // upper 63:62 bits unused
    reg   [8:0] maint_address/* synthesis preserve = 1 */;
    reg         cfg_rdy_1d/* synthesis preserve = 1 */;
    
    reg         last_packet_beat_1d/* synthesis preserve = 1 */;
    reg         last_packet_beat_2d/* synthesis preserve = 1 */;
    reg         last_packet_beat_3d/* synthesis preserve = 1 */;
    reg  [15:0] instruction_cnt/* synthesis preserve = 1 */;
    wire [32:0] data_mask/* synthesis syn_keep=1 */;///
    reg         srio_trac_start_1d/* synthesis preserve = 1 */;
    reg         srio_trac_start_2d/* synthesis preserve = 1 */;
    reg         srio_trac_start_3d/* synthesis preserve = 1 */;
    reg         maint_done_1d/* synthesis preserve = 1 */;     
    reg         maint_done_2d/* synthesis preserve = 1 */;     
    reg         srio_trac_test/* synthesis preserve = 1 */;
    
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
///process   
    assign cfg_wdata_o = maint_data_out[35:4];
    assign cfg_addr_o  = maint_data_out[58:37];
    assign maint_data_out_d = instruction[maint_address];
    assign data_mask   = { {8{maint_data_out[3]}}, {8{maint_data_out[2]}},
                           {8{maint_data_out[1]}}, {8{maint_data_out[0]}} };
    
    genvar ii;
    generate
        for (ii = 0; ii < 512; ii = ii + 1) begin : instruction_gen
            // Load MAINTs  
            if(ii < NUMBER_OF_INSTRUCTIONS) begin
                assign  instruction[ii] = maintenance_instruction[(ii+1)*64-1:ii*64];
            end
            else begin
                assign  instruction[ii] = 64'h0;
            end
        end
    endgenerate

    always @(posedge clk_i) begin
        srio_trac_start_1d <= srio_trac_start_i;
        srio_trac_start_2d <= srio_trac_start_1d;
        srio_trac_start_3d <= srio_trac_start_2d;
        maint_done_1d      <= maint_done_o;
        maint_done_2d      <= maint_done_1d;
    end

    always @(posedge clk_i) begin
        if (rst_i) begin
            srio_trac_test <= 1'b0;
        end else if (srio_trac_start_2d && (!srio_trac_start_3d)) begin
            srio_trac_test <= 1'b1;
        end else if (maint_done_2d) begin
            srio_trac_test <= 1'b0;
        end
    end
    
    always @(posedge clk_i) begin
        if (rst_i) begin
            last_packet_beat_1d      <= 1'b1;
            last_packet_beat_2d      <= 1'b1;
            last_packet_beat_3d      <= 1'b1;
        end else begin
            last_packet_beat_1d      <= cfg_rdy_i;
            last_packet_beat_2d      <= last_packet_beat_1d;
            last_packet_beat_3d      <= last_packet_beat_2d || (~srio_trac_test);
        end
    end
    
    always @(posedge clk_i) begin
        if (rst_i) begin
            maint_address <= STARTING_ADDRESS;
        end
        else if(maint_done_2d) begin
            maint_address <= STARTING_ADDRESS;
        end
        else if (cfg_rdy_i) begin
            maint_address <= maint_address + 9'd1;
        end
    end
    
    always @(posedge clk_i) begin
        if (rst_i) begin
            cfg_wr_o         <= 1'b0;
            cfg_sel_o        <= 1'b0;
            instruction_cnt  <= 16'h0;
        end
        else if (maint_done_2d) begin
            cfg_wr_o         <= 1'b0;
            cfg_sel_o        <= 1'b0;
            instruction_cnt  <= 16'h0;
        end 
        else if (srio_trac_test &&
                     (instruction_cnt < NUMBER_OF_INSTRUCTIONS && last_packet_beat_3d)) begin
            cfg_wr_o         <= maint_data_out[36] == WRITE;
            cfg_sel_o        <= 1'b1;
        end 
        else if (cfg_rdy_i) begin
            cfg_wr_o         <= 1'b0;
            cfg_sel_o        <= 1'b0;
            instruction_cnt  <= instruction_cnt + 16'h1;
        end 
        else if (instruction_cnt == NUMBER_OF_INSTRUCTIONS) begin
            cfg_wr_o         <= 1'b0;
            cfg_sel_o        <= 1'b0;
        end
    end
    
    always @(posedge clk_i) begin
        if (rst_i) begin
            cfg_ena_o         <= 1'b0;
        end 
        else if (cfg_rdy_i == 1'b1) begin
            cfg_ena_o         <= 1'b0;
        end
        else if (cfg_sel_o == 1'b1) begin
            cfg_ena_o         <= 1'b1;
        end 
    end
    
    always @ (posedge clk_i) begin
        maint_data_out  <= maint_data_out_d[61:0];
        cfg_rdy_1d      <= cfg_rdy_i;
    end
    
    always @ (posedge clk_i) begin
        if (rst_i) begin
            maint_autocheck_error_o <= 1'b0;
        end 
        else if (!srio_trac_start_2d && (srio_trac_start_3d)) begin
            maint_autocheck_error_o <= 1'b0;
        end 
        else if (cfg_rdy_i && maint_data_out[36] == READ) begin
            if ((cfg_wdata_o & data_mask) != (cfg_rdata_i & data_mask)) begin
                maint_autocheck_error_o <= 1'b1;
            end 
///            else begin
///                maint_autocheck_error_o <= 1'b0;
///            end
        end
    end
    
    always @ (posedge clk_i) begin
        if (rst_i) begin
            maint_done_o <= 1'b0;
        end 
        else if (!srio_trac_start_2d && (srio_trac_start_3d)) begin
            maint_done_o <= 1'b0;
        end 
        else if ((instruction_cnt == NUMBER_OF_INSTRUCTIONS) &&
                     (cfg_rdy_1d == 1'b1)) begin
             maint_done_o <= 1'b1;
        end
    end


endmodule
