                                
`timescale 1ps/1ps

module gowin_srio_request_gen
   (
    input             clk_i,
    input             rst_i,

    input      [15:0] deviceid_i,
    input      [15:0] dest_id_i,

    output            tx_req_tvalid_o,
    input             tx_req_tready_i,
    output            tx_req_tlast_o,
    output     [63:0] tx_req_tdata_o,
    output reg [31:0] tx_req_tidinfo_o/* synthesis preserve = 1 */,

    input             rx_resp_tvalid_i,
    output reg        rx_resp_tready_o/* synthesis preserve = 1 */,
    input             rx_resp_tlast_i,
    input      [63:0] rx_resp_tdata_i,
    input      [31:0] rx_resp_tidinfo_i,

    input             srio_trac_start_i,

    output            request_autocheck_error_o,
    output            msg_autocheck_error_o,
    output            request_done_o
   )/* synthesis syn_srlstyle = "registers" */;

//////////////////////////////////////////////////////////////////////////////////////////////////
///parameter
    localparam [3:0] NREAD                  = 4'h2;
    localparam [3:0] NWRITE                 = 4'h5;
    localparam [3:0] SWRITE                 = 4'h6;
    localparam [3:0] DOORB                  = 4'hA;
    localparam [3:0] MESSG                  = 4'hB;
    localparam [3:0] RESP                   = 4'hD;
                                            
    localparam [3:0] TNWR                   = 4'h4;
    localparam [3:0] TNWR_R                 = 4'h5;
    localparam [3:0] TNRD                   = 4'h4;
                                            
    localparam [3:0] TNDATA                 = 4'h0;
    localparam [3:0] MSGRSP                 = 4'h1;
    localparam [3:0] TWDATA                 = 4'h8;
    
    localparam       NUM_SWRITES            = 37;   // 37 
    localparam       NUM_NWRITERS           = 19; // 19:0
    localparam       NUM_NWRITES            = 19;
    localparam       NUM_NREADS             = 26; // 26
    localparam       NUM_DBS                = 2 ;
    localparam       NUM_MSGS               = 17;
    localparam       NUMBER_OF_INSTRUCTIONS = NUM_SWRITES + NUM_NWRITERS + NUM_NWRITES + NUM_NREADS + NUM_DBS + NUM_MSGS;
  
    localparam [64*NUM_SWRITES-1:0] swrite_instruction = {
      // RSVD,   FTYPE, TTYPE,  ADDRESS,       SIZE
      // SWRITEs
      {12'h000, SWRITE, 4'h0,   36'h000000777, 8'd0},  
      {12'h000, SWRITE, 4'h0,   36'h000008806, 8'd0},  
      {12'h000, SWRITE, 4'h0,   36'h000000125, 8'd0},  
      {12'h000, SWRITE, 4'h0,   36'h000000124, 8'd0},  
      {12'h000, SWRITE, 4'h0,   36'h000000123, 8'd0},  
      {12'h000, SWRITE, 4'h0,   36'h000000122, 8'd0},  
      {12'h000, SWRITE, 4'h0,   36'h000000121, 8'd0},  
      {12'h000, SWRITE, 4'h0,   36'h000000120, 8'd0},  
      {12'h000, SWRITE, 4'h0,   36'h000000126, 8'd1},  
      {12'h000, SWRITE, 4'h0,   36'h000000124, 8'd1},  
      {12'h000, SWRITE, 4'h0,   36'h000000122, 8'd1},  
      {12'h000, SWRITE, 4'h0,   36'h000004350, 8'd1},  
      {12'h000, SWRITE, 4'h0,   36'h000004355, 8'd2},  
      {12'h000, SWRITE, 4'h0,   36'h000012300, 8'd2},  
      {12'h000, SWRITE, 4'h0,   36'h000012304, 8'd3},  
      {12'h000, SWRITE, 4'h0,   36'h000345000, 8'd3},  
      {12'h000, SWRITE, 4'h0,   36'h000345003, 8'd4},  
      {12'h000, SWRITE, 4'h0,   36'h004550000, 8'd4},  
      {12'h000, SWRITE, 4'h0,   36'h004550002, 8'd5},  
      {12'h000, SWRITE, 4'h0,   36'h198877600, 8'd5},  
      {12'h000, SWRITE, 4'h0,   36'h198877601, 8'd6},  
      {12'h000, SWRITE, 4'h0,   36'h2ABBCCDD8, 8'd6},  
      {12'h000, SWRITE, 4'h0,   36'h2ABBCCDD8, 8'd7},  
      {12'h000, SWRITE, 4'h0,   36'h2ABBCCDD8, 8'd15}, 
      {12'h000, SWRITE, 4'h0,   36'h2ABBCCDD8, 8'd31}, 
      {12'h000, SWRITE, 4'h0,   36'h120000600, 8'd63}, 
      {12'h000, SWRITE, 4'h0,   36'h230000600, 8'd95}, 
      {12'h000, SWRITE, 4'h0,   36'h340000600, 8'd127},
      {12'h000, SWRITE, 4'h0,   36'h450000600, 8'd255},
      {12'h000, SWRITE, 4'h0,   36'h560000600, 8'd15}, 
      {12'h000, SWRITE, 4'h0,   36'h670000600, 8'd31}, 
      {12'h000, SWRITE, 4'h0,   36'h780000600, 8'd63}, 
      {12'h000, SWRITE, 4'h0,   36'h780000600, 8'd95}, 
      {12'h000, SWRITE, 4'h0,   36'h890000600, 8'd127},
      {12'h000, SWRITE, 4'h0,   36'h9A0000600, 8'd255},
      {12'h000, SWRITE, 4'h0,   36'hAB0000600, 8'd1},   
      {12'h000, SWRITE, 4'h0,   36'hCD0000600, 8'd79}}; 
    
    localparam [64*NUM_NWRITERS-1:0] nwriter_instruction = {
      // NWRITE_R
      {12'h000, NWRITE, TNWR_R, 36'h000000777, 8'd0},
      {12'h000, NWRITE, TNWR_R, 36'h000008806, 8'd0},
      {12'h000, NWRITE, TNWR_R, 36'h000000125, 8'd0},
      {12'h000, NWRITE, TNWR_R, 36'h000000124, 8'd0},
      {12'h000, NWRITE, TNWR_R, 36'h000000123, 8'd0},
      {12'h000, NWRITE, TNWR_R, 36'h000000122, 8'd0},
      {12'h000, NWRITE, TNWR_R, 36'h000000121, 8'd0},
      {12'h000, NWRITE, TNWR_R, 36'h000000120, 8'd0},
      {12'h000, NWRITE, TNWR_R, 36'h000000126, 8'd1},
      {12'h000, NWRITE, TNWR_R, 36'h000000124, 8'd1},
      {12'h000, NWRITE, TNWR_R, 36'h000000122, 8'd1},
      {12'h000, NWRITE, TNWR_R, 36'h000004350, 8'd1},
      {12'h000, NWRITE, TNWR_R, 36'h000004355, 8'd2},
      {12'h000, NWRITE, TNWR_R, 36'h000012300, 8'd2},
      {12'h000, NWRITE, TNWR_R, 36'h000012304, 8'd3},
      {12'h000, NWRITE, TNWR_R, 36'h000345000, 8'd255},  
      {12'h000, NWRITE, TNWR_R, 36'h000345003, 8'd4},    
      {12'h000, NWRITE, TNWR_R, 36'h004550000, 8'd255},  
      {12'h000, NWRITE, TNWR_R, 36'h004550000, 8'd71}};   
      
    localparam [64*NUM_NWRITES-1:0] nwrite_instruction = {
      // NWRITE
      {12'h000, NWRITE, TNWR,   36'h198877600, 8'd5},
      {12'h000, NWRITE, TNWR,   36'h198877601, 8'd6},
      {12'h000, NWRITE, TNWR,   36'h2ABBCCDD8, 8'd6},
      {12'h000, NWRITE, TNWR,   36'h2ABBCCDD8, 8'd7},
      {12'h000, NWRITE, TNWR,   36'h2ABBCCDD8, 8'd15},
      {12'h000, NWRITE, TNWR,   36'h2ABBCCDD8, 8'd31},
      {12'h000, NWRITE, TNWR,   36'h120000600, 8'd63},
      {12'h000, NWRITE, TNWR,   36'h230000600, 8'd95},
      {12'h000, NWRITE, TNWR,   36'h340000600, 8'd127},
      {12'h000, NWRITE, TNWR,   36'h450000600, 8'd255},
      {12'h000, NWRITE, TNWR,   36'h560000600, 8'd15},
      {12'h000, NWRITE, TNWR,   36'h670000600, 8'd31},
      {12'h000, NWRITE, TNWR,   36'h780000600, 8'd63},
      {12'h000, NWRITE, TNWR,   36'h890000600, 8'd95},
      {12'h000, NWRITE, TNWR,   36'h9A0000600, 8'd127},
      {12'h000, NWRITE, TNWR,   36'hAB0000600, 8'd255},
      {12'h000, NWRITE, TNWR,   36'hBC0000600, 8'd15},
      {12'h000, NWRITE, TNWR,   36'hCD0000600, 8'd15},
      {12'h000, NWRITE, TNWR,   36'hDE0000600, 8'd15}};
    
    localparam [64*NUM_NREADS-1:0] nread_instruction = {
      // NREADs
      {12'h000, NREAD,  TNRD,   36'h000002307, 8'd00},
      {12'h000, NREAD,  TNRD,   36'h000002406, 8'd00},
      {12'h000, NREAD,  TNRD,   36'h000002505, 8'd00},
      {12'h000, NREAD,  TNRD,   36'h000002604, 8'd00},
      {12'h000, NREAD,  TNRD,   36'h000002703, 8'd00},
      {12'h000, NREAD,  TNRD,   36'h000002802, 8'd00},
      {12'h000, NREAD,  TNRD,   36'h000002301, 8'd00},
      {12'h000, NREAD,  TNRD,   36'h000002400, 8'd00},
      {12'h000, NREAD,  TNRD,   36'h000002506, 8'd01},
      {12'h000, NREAD,  TNRD,   36'h000002604, 8'd01},
      {12'h000, NREAD,  TNRD,   36'h000002702, 8'd01},
      {12'h000, NREAD,  TNRD,   36'h000002800, 8'd01},
      {12'h000, NREAD,  TNRD,   36'h000002305, 8'd02},
      {12'h000, NREAD,  TNRD,   36'h000002400, 8'd02},
      {12'h000, NREAD,  TNRD,   36'h000002504, 8'd03},
      {12'h000, NREAD,  TNRD,   36'h000002600, 8'd03},
      {12'h000, NREAD,  TNRD,   36'h000002703, 8'd04},
      {12'h000, NREAD,  TNRD,   36'h000002800, 8'd04},
      {12'h000, NREAD,  TNRD,   36'h000002502, 8'd05},
      {12'h000, NREAD,  TNRD,   36'h000002600, 8'd05},
      {12'h000, NREAD,  TNRD,   36'h000002701, 8'd06},
      {12'h000, NREAD,  TNRD,   36'h000002800, 8'd06},
      {12'h000, NREAD,  TNRD,   36'h0000023F0, 8'd07},
      {12'h000, NREAD,  TNRD,   36'h000002400, 8'd15},
      {12'h000, NREAD,  TNRD,   36'h000002500, 8'd31},
      {12'h000, NREAD,  TNRD,   36'h200002600, 8'd63}};
    
    localparam [64*NUM_DBS-1:0] db_instruction = {
      // DOORBELLs
      {12'h000, DOORB,  4'b0,   36'h0DBDB0000, 8'd01},
      {12'h000, DOORB,  4'b0,   36'h044440000, 8'd01}};  
    
    
    localparam [64*NUM_MSGS-1:0] msg_instruction = {
      // MESSAGEs
      {12'h0ff, MESSG,  4'b0,   36'h000000002, 8'd07},
      {12'h0fe, MESSG,  4'b0,   36'h000000012, 8'd07},
      {12'h0ed, MESSG,  4'b0,   36'h000000022, 8'd07},
      {12'h0dc, MESSG,  4'b0,   36'h000000002, 8'd07},
      {12'h0cb, MESSG,  4'b0,   36'h000000012, 8'd07},
      {12'h0ba, MESSG,  4'b0,   36'h000000022, 8'd07},
      {12'h0a9, MESSG,  4'b0,   36'h000000002, 8'd07},
      {12'h098, MESSG,  4'b0,   36'h000000012, 8'd07},
      {12'h087, MESSG,  4'b0,   36'h000000022, 8'd07},
      {12'h076, MESSG,  4'b0,   36'h000000002, 8'd07},
      {12'h065, MESSG,  4'b0,   36'h000000012, 8'd07},
      {12'h054, MESSG,  4'b0,   36'h000000022, 8'd07},
      {12'h043, MESSG,  4'b0,   36'h000000002, 8'd07},
      {12'h032, MESSG,  4'b0,   36'h000000012, 8'd07},
      {12'h021, MESSG,  4'b0,   36'h000000022, 8'd07},
      {12'h010, MESSG,  4'b0,   36'h000000002, 8'd07},
      {12'h000, MESSG,  4'b0,   36'h000000012, 8'd16}};  

    localparam     S_IDLE    = 3'b000;
    localparam     S_RD      = 3'b001;
    localparam     S_WAIT    = 3'b010;
    localparam     S_HEAD    = 3'b011;
    localparam     S_TDATA   = 3'b100;
    
//////////////////////////////////////////////////////////////////////////////////////////////////
///signal   
    reg [2:0]      state = S_IDLE/* synthesis preserve = 1 */;
    reg [2:0]      next_state/* synthesis preserve = 1 */;       

    wire [63:0]    instruction [0:511];
 
    reg  [15:0]    src_id/* synthesis syn_keep=1 */;
    reg  [1:0]     prio/* synthesis syn_keep=1 */;
    reg  [7:0]     tid/* synthesis syn_keep=1 */;
    reg  [35:0]    srio_addr/* synthesis syn_keep=1 */;
    reg  [63:0]    header_beat=0/* synthesis syn_keep=1 */;
    reg  [3:0]     current_ftype/* synthesis preserve = 1 */;
    reg  [3:0]     current_ttype/* synthesis preserve = 1 */;
    reg  [7:0]     current_size/* synthesis preserve = 1 */;
    reg  [7:0]     msg/* synthesis preserve = 1 */;
    reg  [4:0]     tdata_size=0/* synthesis preserve = 1 */;
    reg            tdata_start=0/* synthesis preserve = 1 */;
    reg            tdata_last=0/* synthesis preserve = 1 */;
    reg  [63:0]    tdata_out/* synthesis preserve = 1 */;
    reg  [7:0]     increase_data=0/* synthesis preserve = 1 */;
    reg  [7:0]     increase_data_1d=0/* synthesis preserve = 1 */;

    reg            srio_trac_start_1d/* synthesis preserve = 1 */;     
    reg            srio_trac_start_2d/* synthesis preserve = 1 */;     
    reg            request_done_1d/* synthesis preserve = 1 */;        
    reg            request_done_2d/* synthesis preserve = 1 */;        
    reg            request_done_3d/* synthesis preserve = 1 */;        
    reg            srio_trac_test_delay_1d/* synthesis preserve = 1 */;
    reg            srio_trac_test_delay_2d/* synthesis preserve = 1 */;
    reg            fifo_afull_1d/* synthesis preserve = 1 */;          
    reg            fifo_afull_2d/* synthesis preserve = 1 */;          
    reg            fifo_afull_3d/* synthesis preserve = 1 */;          
                   
    reg  [12:0]    srio_trac_test_cnt=0/* synthesis preserve = 1 */;
    wire           srio_trac_test_delay/* synthesis syn_keep=1 */;
    reg            srio_trac_test/* synthesis preserve = 1 */;
    
    reg            test_en/* synthesis preserve = 1 */;
    reg  [8:0]     request_address/* synthesis preserve = 1 */;
    reg  [63:0]    request_data_out/* synthesis preserve = 1 */;
    reg  [15:0]    instruction_cnt/* synthesis preserve = 1 */;
    reg            sent_all_packets/* synthesis preserve = 1 */;
    reg  [63:0]    tx_req_tdata_1d/* synthesis preserve = 1 */; 
    reg            tx_req_tvalid_1d/* synthesis preserve = 1 */;
    reg            tx_req_tlast_1d/* synthesis preserve = 1 */; 
    reg  [3:0]     tx_ftype_1d/* synthesis preserve = 1 */;   
    reg  [3:0]     tx_ttype_1d/* synthesis preserve = 1 */;    
    reg  [7:0]     msg_1d/* synthesis preserve = 1 */;          
    reg  [7:0]     tid_1d/* synthesis preserve = 1 */;          
    reg  [7:0]     tx_size_1d/* synthesis preserve = 1 */;      
    reg            fifo_wen=0/* synthesis preserve = 1 */;
    reg  [95:0]    fifo_wdata/* synthesis preserve = 1 */;
    wire           fifo_afull/* synthesis syn_keep=1 */;
    wire           fifo_empty/* synthesis syn_keep=1 */;
    wire [95:0]    fifo_rdata/* synthesis syn_keep=1 */;
    reg            fifo_ren/* synthesis preserve = 1 */;
    reg  [7:0]     expected_tid=0/* synthesis preserve = 1 */;
    reg  [3:0]     expected_ftype=0/* synthesis preserve = 1 */;
    reg  [7:0]     expected_size=0/* synthesis preserve = 1 */;
    reg  [7:0]     expected_msg_info=0/* synthesis preserve = 1 */;
    reg  [7:0]     rx_tid=0/* synthesis preserve = 1 */;
    reg  [3:0]     rx_ttype=0/* synthesis preserve = 1 */;
    reg  [3:0]     rx_ftype=0/* synthesis preserve = 1 */;
    reg  [7:0]     rx_msg_info=0/* synthesis preserve = 1 */;
    reg            fifo_ren_1d/* synthesis preserve = 1 */;
    reg            fifo_ren_2d/* synthesis preserve = 1 */;
    reg            rx_resp_tvalid/* synthesis preserve = 1 */;
    reg            rx_resp_tlast/* synthesis preserve = 1 */;
    reg [63:0]     rx_resp_tdata/* synthesis preserve = 1 */;
    reg            rx_resp_tvalid_1d/* synthesis preserve = 1 */;
    reg            rx_resp_tlast_1d/* synthesis preserve = 1 */;
    reg [63:0]     rx_resp_tdata_1d/* synthesis preserve = 1 */;
    reg            rx_resp_tvalid_2d/* synthesis preserve = 1 */;
    reg            rx_resp_tlast_2d/* synthesis preserve = 1 */;
    reg [63:0]     rx_resp_tdata_2d/* synthesis preserve = 1 */;
    reg            rx_resp_start/* synthesis preserve = 1 */;
    reg            fifo_wen_1d/* synthesis preserve = 1 */;
    reg            fifo_wen_2d/* synthesis preserve = 1 */;
    reg [8:0]      fifo_wr_cnt/* synthesis preserve = 1 */;
    reg [8:0]      fifo_rd_cnt/* synthesis preserve = 1 */;
    
    reg            rd_flag/* synthesis preserve = 1 */;
    reg            request_autocheck_error/* synthesis preserve = 1 */;
    reg            msg_autocheck_error/* synthesis preserve = 1 */;
    reg            request_done/* synthesis preserve = 1 */;
    reg            request_autocheck_error_1d/* synthesis preserve = 1 */;
    reg            request_autocheck_error_2d/* synthesis preserve = 1 */;
    reg            msg_autocheck_error_1d/* synthesis preserve = 1 */;    
    reg            msg_autocheck_error_2d/* synthesis preserve = 1 */;  
    
    reg            tx_req_state_end/* synthesis preserve = 1 */; 
    reg            no_payload/* synthesis preserve = 1 */; 
    reg            head_trans/* synthesis preserve = 1 */; 
 
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
///process
    // instruction list
    genvar i;
    generate
        for (i = 0; i < 512; i = i + 1) begin : instruction_gen
            // SWRITES
            if(i < NUM_SWRITES) begin
               assign  instruction[i]  = swrite_instruction[(i+1)*64-1:(i)*64];
            // NWRITE_Rs
            end 
            else if(i < (NUM_SWRITES + NUM_NWRITERS)) begin
               assign  instruction[i]  = nwriter_instruction[(i-NUM_SWRITES+1)*64-1:(i-NUM_SWRITES)*64];
            // NWRITEs
            end 
            else if(i < (NUM_SWRITES + NUM_NWRITERS + NUM_NWRITES)) begin
               assign  instruction[i]  = nwrite_instruction[(i-NUM_SWRITES-NUM_NWRITERS+1)*64-1:(i-NUM_SWRITES-NUM_NWRITERS)*64];
            // NREADs
            end 
            else if(i < (NUM_SWRITES + NUM_NWRITERS + NUM_NWRITES + NUM_NREADS)) begin
               assign  instruction[i]  = nread_instruction[(i-NUM_SWRITES-NUM_NWRITERS-NUM_NWRITES+1)*64-1:(i-NUM_SWRITES-NUM_NWRITERS-NUM_NWRITES)*64];
            // DBs
            end 
            else if(i < (NUM_SWRITES + NUM_NWRITERS + NUM_NWRITES + NUM_NREADS + NUM_DBS)) begin
               assign  instruction[i]  = db_instruction[(i-NUM_SWRITES-NUM_NWRITERS-NUM_NWRITES-NUM_NREADS+1)*64-1:(i-NUM_SWRITES-NUM_NWRITERS-NUM_NWRITES-NUM_NREADS)*64];
            // MSGs
            end 
            else if(i < (NUM_SWRITES + NUM_NWRITERS + NUM_NWRITES + NUM_NREADS + NUM_DBS + NUM_MSGS)) begin
               assign  instruction[i]  = msg_instruction[(i-NUM_SWRITES-NUM_NWRITERS-NUM_NWRITES-NUM_NREADS-NUM_DBS+1)*64-1:(i-NUM_SWRITES-NUM_NWRITERS-NUM_NWRITES-NUM_NREADS-NUM_DBS)*64];
            end 
            else begin
               assign  instruction[i]  = 64'h0;
            end
        end
    endgenerate
    
    always@(posedge clk_i) begin
        if(rst_i)
            state <= S_IDLE;
        else 
            state <= next_state;
    end
    
    always@(*) begin
        case(state)
            S_IDLE : 
                begin
                    if(test_en == 1'b1)
                        next_state = S_RD;
                    else 
                        next_state = S_IDLE;
                end
            
            S_RD :
                begin
                    next_state = S_WAIT;
                end
    
            S_WAIT :
                begin
                    next_state = S_HEAD;
                end
    
            S_HEAD :
                begin
                    if(((current_ftype == NREAD) || (current_ftype == DOORB)) && (tx_req_tvalid_o == 1'b1) && (tx_req_tready_i == 1'b1))
                        next_state = S_IDLE;
                    else if((tx_req_tvalid_o == 1'b1) && (tx_req_tready_i == 1'b1))
                        next_state = S_TDATA;
                    else 
                        next_state = S_HEAD;
                end
    
            S_TDATA :
                begin
                    if((tx_req_tvalid_o == 1'b1) && (tx_req_tready_i == 1'b1) && (tx_req_tlast_o == 1'b1))
                        next_state = S_IDLE;
                    else 
                        next_state = S_TDATA;
                end
    
            
            default :
                begin
                    next_state = S_IDLE;
                end
        endcase
    end
    
    assign tx_req_tvalid_o           = ((state == S_TDATA) | (state == S_HEAD)) ? 1'b1 : 1'b0;
    assign tx_req_tlast_o            = tdata_last;
    assign tx_req_tdata_o            = tdata_out;
    assign srio_trac_test_delay      = srio_trac_test_cnt[2];///srio_trac_test_cnt[12];
    assign request_autocheck_error_o = request_autocheck_error_2d;
    assign msg_autocheck_error_o     = msg_autocheck_error_2d;
    assign request_done_o            = request_done_2d;
    
    always @(posedge clk_i) begin
        if (rst_i) begin
            tx_req_state_end         <= 1'b0;
        end
        else if((state == S_HEAD) && ((current_ftype == NREAD) || (current_ftype == DOORB)) && (tx_req_tvalid_o == 1'b1) && (tx_req_tready_i == 1'b1)) begin
            tx_req_state_end         <= 1'b1;
        end
        else if((state == S_TDATA) & (tx_req_tvalid_o == 1'b1) && (tx_req_tready_i == 1'b1) && (tx_req_tlast_o == 1'b1)) begin
            tx_req_state_end         <= 1'b1;
        end
        else begin
            tx_req_state_end         <= 1'b0;
        end
    end

    
    always @(posedge clk_i) begin
        srio_trac_start_1d         <= srio_trac_start_i;
        srio_trac_start_2d         <= srio_trac_start_1d;
        request_done_1d            <= request_done;
        request_done_2d            <= request_done_1d;
        request_done_3d            <= request_done_2d;
        srio_trac_test_delay_1d    <= srio_trac_test_delay;
        srio_trac_test_delay_2d    <= srio_trac_test_delay_1d;
        fifo_afull_1d              <= fifo_afull;
        fifo_afull_2d              <= fifo_afull_1d;
        fifo_afull_3d              <= fifo_afull_2d;
        request_autocheck_error_1d <= request_autocheck_error;
        request_autocheck_error_2d <= request_autocheck_error_1d;
        msg_autocheck_error_1d     <= msg_autocheck_error;
        msg_autocheck_error_2d     <= msg_autocheck_error_1d;  
    end

    always @(posedge clk_i) begin
        if (rst_i) begin
            srio_trac_test <= 1'b0;
        end 
        else if (srio_trac_start_1d && (!srio_trac_start_2d)) begin
            srio_trac_test <= 1'b1;
        end 
        else if (request_done) begin
            srio_trac_test <= 1'b0;
        end
    end

    always @(posedge clk_i) begin
       if (srio_trac_test && !srio_trac_test_delay) begin
            srio_trac_test_cnt <= srio_trac_test_cnt + 1'b1;
        end 
        else if (!srio_trac_test) begin
            srio_trac_test_cnt <= 0;
        end
    end

    always @(posedge clk_i) begin
        if (rst_i) begin
            test_en <= 1'b0;
        end 
        else if ((tx_req_tready_i == 1'b1) && (sent_all_packets == 1'b0) && (srio_trac_test_delay_1d == 1'b1) && (fifo_afull_3d == 1'b0)) begin
            test_en <= 1'b1;
        end 
        else begin
            test_en <= 1'b0;
        end
    end
    
    always @(posedge clk_i) begin
        if (rst_i) begin
            request_address <= 9'h0;
        end 
        else if (request_done_2d == 1'b1) begin
            request_address <= 9'h0;
        end 
        else if ((state == S_IDLE) && (test_en == 1'b1)) begin
            request_address <= request_address + 1'b1;
        end
    end
    
    always @(posedge clk_i) begin
        request_data_out       <= instruction[request_address];
    end

    always @(posedge clk_i) begin
        if(state == S_IDLE) begin
            src_id                 <= deviceid_i;
            prio                   <= 2'h1;
            tx_req_tidinfo_o       <= {src_id, dest_id_i};
            tid                    <= request_address[7:0];
            srio_addr              <= request_data_out[43:8];
            current_ftype          <= request_data_out[51:48];
            current_ttype          <= request_data_out[47:44];
            current_size           <= request_data_out[7:0];
            msg                    <= request_data_out[59:52]; 
        end
    end

    always @(posedge clk_i) begin
        if((state == S_IDLE) && (test_en == 1'b1)) begin
            rd_flag                 <= 1'b1;
        end
        else begin
            rd_flag                 <= 1'b0;
        end
    end

    always @(posedge clk_i) begin
        if(rd_flag == 1'b1) begin
            if(current_ftype == MESSG) begin
                header_beat            <= {msg,current_ftype,current_ttype,1'b0, prio, 1'b0, current_size, srio_addr};
            end
            else begin
                header_beat            <= {tid,current_ftype,current_ttype,1'b0, prio, 1'b0, current_size, srio_addr};
            end
        end
    end
        
    always @(posedge clk_i) begin
        if(state == S_WAIT) begin
            tdata_size   <= current_size[7:3];
        end
        else if((state == S_TDATA) && (tx_req_tready_i == 1'b1) && (tx_req_tvalid_o == 1'b1) && (tdata_size > 5'd0)) begin
            tdata_size   <= tdata_size - 5'd1;
        end
    end
    
    always @(posedge clk_i) begin
        if(state == S_RD) begin
            tdata_start   <= 1'b1;
        end
        else if((tdata_start == 1'b1) && (tx_req_tready_i == 1'b1) && (tx_req_tvalid_o == 1'b1)) begin
            tdata_start   <= 1'b0;
        end
    end
    
    always @(posedge clk_i) begin
        if(rst_i) begin
            tdata_last   <= 1'b1;
        end
        else if((state == S_WAIT) && ((current_ftype == NREAD) || (current_ftype == DOORB))) begin
            tdata_last   <= 1'b1;
        end
        else if(state == S_WAIT) begin
            tdata_last   <= 1'b0;
        end
        else if((tdata_last == 1'b1) && (tx_req_tready_i == 1'b1) && (tx_req_tvalid_o == 1'b1)) begin
            tdata_last   <= 1'b0;
        end
        else if((state == S_TDATA) && (tx_req_tready_i == 1'b1) && (tx_req_tvalid_o == 1'b1) && (tdata_size == 5'd1)) begin
            tdata_last   <= 1'b1;
        end
        else if((state == S_HEAD) && (current_ftype != NREAD) && (current_ftype != DOORB) && (tx_req_tready_i == 1'b1) && (tx_req_tvalid_o == 1'b1) && (tdata_size == 5'd0)) begin
            tdata_last   <= 1'b1;
        end
    end
    
    always @(posedge clk_i) begin
        if((tdata_start == 1'b1) && ((tx_req_tready_i == 1'b0) || (tx_req_tvalid_o == 1'b0)))begin
            tdata_out   <= header_beat;
        end
        else begin
            tdata_out   <= {8{increase_data}};
        end
    end
    
    always @* begin
        increase_data = increase_data_1d;
        if((tdata_start == 1'b0) && (tx_req_tready_i == 1'b1) && (tx_req_tvalid_o == 1'b1)) begin
            increase_data   = increase_data_1d + 8'h1;
        end
    end
    
    always @(posedge clk_i) begin
        if(request_done_2d == 1'b1) begin
            increase_data_1d   <= 8'h00;   
        end
        else begin
            increase_data_1d   <= increase_data;
        end
    end
    
    always @(posedge clk_i) begin
        if (rst_i) begin
            instruction_cnt  <= 16'h0;
            sent_all_packets <= 1'b0;
        end 
        else if (request_done_3d == 1'b1) begin
            instruction_cnt  <= 16'h0;
            sent_all_packets <= 1'b0;
        end 
        else if (tx_req_state_end == 1'b1) begin
            instruction_cnt  <= instruction_cnt + 1'b1;
            sent_all_packets <= sent_all_packets;
        end 
        else if (instruction_cnt == NUMBER_OF_INSTRUCTIONS) begin
            sent_all_packets <= 1'b1;
            instruction_cnt  <= instruction_cnt;
        end
    end

    always @(posedge clk_i) begin
        tx_req_tdata_1d   <= tx_req_tdata_o;
        tx_req_tvalid_1d  <= (tx_req_tready_i == 1'b1) && (tx_req_tvalid_o == 1'b1);
        tx_req_tlast_1d   <= (tx_req_tready_i == 1'b1) && (tx_req_tvalid_o == 1'b1) && (tx_req_tlast_o == 1'b1);
        tx_ftype_1d       <= current_ftype;
        tx_ttype_1d       <= current_ttype;
        msg_1d            <= msg;
        tid_1d            <= tid;
        tx_size_1d        <= current_size;
    end
    
   ///need reponse
    always @(posedge clk_i) begin
        if (rst_i) begin
            fifo_wen    <= 1'h0;
        end 
        else if (((tx_ftype_1d == NREAD) || (tx_ftype_1d == DOORB) || (tx_ftype_1d == MESSG) 
                  ||((tx_ftype_1d == NWRITE) && (tx_ttype_1d == TNWR_R))) && (tx_req_tlast_1d == 1'b1)) begin
            fifo_wen    <= 1'b1;
        end 
        else begin
            fifo_wen    <= 1'b0;
        end
    end

    always @(posedge clk_i) begin
        if (tx_ftype_1d == MESSG) begin
            fifo_wdata  <= {32'd0,36'b0,msg_1d, tid_1d, tx_ftype_1d, tx_size_1d};
        end 
        else begin
            fifo_wdata  <= {32'd0,36'b0,8'd0, tid_1d, tx_ftype_1d, tx_size_1d};
        end 
    end

fifo_sc_top u_fifo_top
(
///	.WrReset       ( rst_i            ), //input WrReset
///	.RdReset       ( rst_i            ), //input RdReset
///	.WrClk         ( clk_i            ), //input WrClk
///	.RdClk         ( clk_i            ), //input RdClk
    .Clk           ( clk_i            ),
    .Reset         ( rst_i            ),
    
	.WrEn          ( fifo_wen         ), //input WrEn
	.Data          ( fifo_wdata       ), //input [95:0] Data
	.Almost_Full   ( fifo_afull       ), //output Almost_Full
	.Full          (                  ), //output Full
    
	.RdEn          ( fifo_ren         ), //input RdEn
	.Q             ( fifo_rdata       ), //output [95:0] Q
	.Almost_Empty  (                  ), //output Almost_Empty
	.Empty         ( fifo_empty       )  //output Empty

);
    
    always @(posedge clk_i) begin
        if((fifo_empty == 1'b0) && (rx_resp_start == 1'b1) && (rx_resp_tvalid == 1'b1)) begin
            fifo_ren        <= 1'b1;
        end
        else begin
            fifo_ren        <= 1'b0;
        end
    end
    
    always @(posedge clk_i) begin
        if(fifo_ren_1d == 1'b1) begin
            expected_tid        <= fifo_rdata[19:12];
            expected_ftype      <= fifo_rdata[11:8];
            expected_size       <= fifo_rdata[7:0];
            expected_msg_info   <= fifo_rdata[27:20];
        end
    end

    always @(posedge clk_i) begin
        if(fifo_ren_1d == 1'b1) begin
            rx_tid              <= rx_resp_tdata_2d[63:56];
            rx_ttype            <= rx_resp_tdata_2d[51:48];
            rx_ftype            <= rx_resp_tdata_2d[55:52];
            rx_msg_info         <= rx_resp_tdata_2d[63:56];
        end
    end
    
    always @(posedge clk_i) begin
        fifo_ren_1d             <= fifo_ren;
        fifo_ren_2d             <= fifo_ren_1d;
        rx_resp_tvalid          <= rx_resp_tvalid_i & rx_resp_tready_o;
        rx_resp_tlast           <= rx_resp_tlast_i & rx_resp_tvalid_i & rx_resp_tready_o;
        rx_resp_tdata           <= rx_resp_tdata_i;
        rx_resp_tvalid_1d       <= rx_resp_tvalid;
        rx_resp_tlast_1d        <= rx_resp_tlast;
        rx_resp_tdata_1d        <= rx_resp_tdata;
        rx_resp_tvalid_2d       <= rx_resp_tvalid_1d;
        rx_resp_tlast_2d        <= rx_resp_tlast_1d;
        rx_resp_tdata_2d        <= rx_resp_tdata_1d;
    end

    always @(posedge clk_i) begin
        if (rst_i) begin
            rx_resp_tready_o  <= 1'b0;
        end 
        else begin
            rx_resp_tready_o  <= 1'b1;
        end
    end

    always @(posedge clk_i) begin
        if (rst_i) begin
            rx_resp_start  <= 1'b1;
        end 
        else if(request_done_2d == 1'b1) begin
            rx_resp_start  <= 1'b1;
        end
        else if(rx_resp_tlast) begin
            rx_resp_start  <= 1'b1;
        end
        else if(rx_resp_tvalid & (!rx_resp_tlast)) begin
            rx_resp_start  <= 1'b0;
        end
    end

    always @(posedge clk_i) begin
        fifo_wen_1d     <= fifo_wen;
        fifo_wen_2d     <= fifo_wen_1d;
    end

    always @(posedge clk_i) begin
        if (rst_i) begin
            fifo_wr_cnt     <= 9'b0;
        end 
        else if (request_done_2d == 1'b1) begin
            fifo_wr_cnt     <= 9'b0;
        end 
        else if (fifo_wen_2d) begin
            fifo_wr_cnt     <= fifo_wr_cnt + 9'b1;
        end
    end

    always @(posedge clk_i) begin
        if (rst_i) begin
            fifo_rd_cnt     <= 9'b0;
        end 
        else if (request_done_2d == 1'b1) begin
            fifo_rd_cnt     <= 9'b0;
        end 
        else if (fifo_ren_2d) begin
            fifo_rd_cnt     <= fifo_rd_cnt + 9'b1;
        end
    end

    always @(posedge clk_i) begin
        if (rst_i) begin
            request_done     <= 1'b0;
        end 
        else if (!srio_trac_start_1d && (srio_trac_start_2d)) begin
            request_done     <= 1'b0;
        end 
        else if (sent_all_packets && (fifo_wr_cnt <= fifo_rd_cnt)) begin
            request_done     <= 1'b1;
        end
    end

    always @ (posedge clk_i) begin
        if (rst_i) begin
            request_autocheck_error <= 1'b0;
        end 
        else if (!srio_trac_start_1d && (srio_trac_start_2d)) begin
            request_autocheck_error     <= 1'b0;
        end 
        else if (fifo_ren_2d) begin
            if(expected_tid != rx_tid) begin
                // TID mismatch means an error unless it's a message response (MSGs don't use TID)
                if (!(expected_ftype == MESSG) || !(rx_ttype == MSGRSP)) begin
                    request_autocheck_error <= 1'b1;
                    $display ("\t *** TID mismatch Error ***");
                end
            // expecting a Read response
            end 
            else if (expected_ftype == NREAD && !(rx_ttype == TWDATA)) begin
                request_autocheck_error <= 1'b1;
            $display ("\t *** NREAD Read Response Not Received ***");
            // expecting a Response without data
            end 
            else if (expected_ftype == NWRITE && !(rx_ttype == TNDATA) 
                         && rx_ftype !== 4'd13) begin 
                request_autocheck_error <= 1'b1;
            $display ("\t *** NWRITE Response Without Data Error ***");
            // expecting a Response without data
            end 
            else if (expected_ftype == DOORB && !(rx_ttype == TNDATA)) begin
                request_autocheck_error <= 1'b1;
            $display ("\t *** DOORB Response Without Data Error ***");
            end
        end
    end
    
    always@(posedge clk_i) begin
        if(rst_i)
            msg_autocheck_error       <= 1'b0;
        else if (!srio_trac_start_1d && (srio_trac_start_2d)) begin
            msg_autocheck_error       <= 1'b0;
        end 
        else if (fifo_ren_2d) begin
            if(expected_ftype == MESSG) begin
                if(rx_ttype !== MSGRSP) begin
                    msg_autocheck_error <= 1'b1;
                    $display ("\t *** MSG Error0 ***");
                end
                else if(expected_msg_info !== rx_msg_info) begin
                    msg_autocheck_error <= 1'b1;
                    $display ("\t *** MSG Error1 ***");
                end
            end
        end
    end


endmodule

