                               
`timescale 1ps/1ps

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
module gowin_srio_response_gen (
  input             clk_i,
  input             rst_i,

  input      [15:0] deviceid_i,
  input      [15:0] source_id_i,
  
  input             srio_trac_start_i,
  
  input      [8:0]  srio_rx_req_raddr_i,
  output     [63:0] srio_rx_req_rd_data_o,  
  output            srio_rx_req_rd_last_o,  
  output     [31:0] srio_rx_req_rd_idinfo_o,

  output            tx_resp_trac_valid_o,
  input             tx_resp_trac_ready_i,
  output            tx_resp_trac_last_o,
  output     [63:0] tx_resp_trac_data_o,
  output reg [31:0] tx_resp_trac_idinfo_o = 0/* synthesis preserve = 1 */,

  input             rx_req_trac_valid_i,
  output            rx_req_trac_ready_o/* synthesis preserve = 1 */,
  input             rx_req_trac_last_i,
  input      [63:0] rx_req_trac_data_i,
  input      [31:0] rx_req_trac_idinfo_i
  
 )/* synthesis syn_srlstyle = "registers" */;

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
///parameter
  localparam [3:0] NREAD      = 4'h2;
  localparam [3:0] NWRITE     = 4'h5;
  localparam [3:0] SWRITE     = 4'h6;
  localparam [3:0] DOORB      = 4'hA;
  localparam [3:0] MESSG      = 4'hB;
  localparam [3:0] RESP       = 4'hD;
  localparam [3:0] MAINT      = 4'h8;
                              
  localparam [3:0] TNWR       = 4'h4;
  localparam [3:0] TNWR_R     = 4'h5;
  localparam [3:0] TNRD       = 4'h4;
                              
  localparam [3:0] TNDATA     = 4'h0;
  localparam [3:0] MSGRSP     = 4'h1;
  localparam [3:0] TWDATA     = 4'h8;
  localparam [3:0] MAINT_RRSP = 4'h2;
  localparam [3:0] MAINT_WRSP = 4'h3;
  localparam [3:0] MAINT_RREQ = 4'h0;
  localparam [3:0] MAINT_WREQ = 4'h1;

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
///signal
reg                rx_req_valid/* synthesis preserve = 1 */;
reg                rx_req_last/* synthesis preserve = 1 */;
reg                rx_req_start/* synthesis preserve = 1 */;
reg  [63:0]        rx_req_data/* synthesis preserve = 1 */;       
reg  [31:0]        rx_req_idinfo/* synthesis preserve = 1 */;     
reg                srio_trac_start_1d/* synthesis preserve = 1 */;
reg                srio_trac_start_2d/* synthesis preserve = 1 */;
reg                srio_trac_start_3d/* synthesis preserve = 1 */;
reg  [8:0]         rx_req_waddr=0/* synthesis preserve = 1 */;
reg                rx_req_wen/* synthesis preserve = 1 */;
reg  [96:0]        rx_req_din/* synthesis preserve = 1 */;
wire [96:0]        rx_req_dout/* synthesis syn_keep=1 */;

reg                header_valid/* synthesis preserve = 1 */;
reg  [7:0]         current_tid=0/* synthesis preserve = 1 */;
reg  [3:0]         current_ftype=0/* synthesis preserve = 1 */;
reg  [3:0]         current_ttype=0/* synthesis preserve = 1 */;
reg  [7:0]         current_size=0/* synthesis preserve = 1 */;
reg  [1:0]         current_prio=0/* synthesis preserve = 1 */;
reg  [33:0]        current_addr=0/* synthesis preserve = 1 */;
reg  [15:0]        current_srcid=0/* synthesis preserve = 1 */;
reg  [7:0]         current_hopcount=0/* synthesis preserve = 1 */;
reg  [31:0]        current_idinfo=0;/* synthesis preserve = 1 */
reg                fifo_wen/* synthesis preserve = 1 */;
reg  [95:0]        fifo_wdata/* synthesis preserve = 1 */;
wire               fifo_afull/* synthesis syn_keep=1 */;
reg                fifo_ren/* synthesis syn_keep=1 */;
reg                no_payload/* synthesis syn_keep=1 */;
reg                head_trans/* synthesis syn_keep=1 */;
reg                tx_resp_state_end/* synthesis syn_keep=1 */;
wire [95:0]        fifo_rdata/* synthesis syn_keep=1 */;
wire               fifo_empty/* synthesis syn_keep=1 */;
wire [3:0]         responding_ttype/* synthesis syn_keep=1 */;
wire [63:0]        header_beat/* synthesis syn_keep=1 */;
                   
reg  [7:0]         response_tid=0/* synthesis preserve = 1 */;
reg  [3:0]         response_ftype=0/* synthesis preserve = 1 */;
reg  [3:0]         response_ttype=0/* synthesis preserve = 1 */;
reg  [7:0]         response_size=0/* synthesis preserve = 1 */;
reg  [1:0]         response_prio=0/* synthesis preserve = 1 */;
reg  [7:0]         response_hopcount=0/* synthesis preserve = 1 */;
reg  [15:0]        dest_id=0/* synthesis preserve = 1 */;
reg  [8:0]         starting_read_addr=0/* synthesis preserve = 1 */;
reg                pull_from_store=0/* synthesis preserve = 1 */;
reg  [31:0]        response_idinfo=0/* synthesis preserve = 1 */;
reg  [4:0]         tdata_size=0/* synthesis preserve = 1 */;
reg                tdata_start=0/* synthesis preserve = 1 */;
reg                tdata_last=0/* synthesis preserve = 1 */;
reg  [63:0]        tdata_out=0/* synthesis preserve = 1 */;
reg  [7:0]         increase_data=0/* synthesis preserve = 1 */;
reg  [7:0]         increase_data_1d=0/* synthesis preserve = 1 */;

reg  [8:0]         srio_rx_req_raddr_1d/* synthesis preserve = 1 */;
reg  [8:0]         srio_rx_req_raddr_2d/* synthesis preserve = 1 */;
reg  [96:0]        rx_req_dout_1d/* synthesis preserve = 1 */;
reg  [96:0]        rx_req_dout_2d/* synthesis preserve = 1 */;

reg                fifo_nafull_1d/* synthesis preserve = 1 */;
reg                fifo_nafull_2d/* synthesis preserve = 1 */;
reg                fifo_ren_1d/* synthesis preserve = 1 */;

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
///process
///rx requests
assign  rx_req_trac_ready_o     = fifo_nafull_2d;
assign  srio_rx_req_rd_data_o   = rx_req_dout_2d[63:0];
assign  srio_rx_req_rd_last_o   = rx_req_dout_2d[64];  
assign  srio_rx_req_rd_idinfo_o = rx_req_dout_2d[96:65];    

always @(posedge clk_i) begin
    if (rst_i) begin
        rx_req_valid <= 1'b0;
        rx_req_last  <= 1'b0;
    end
    else if(rx_req_trac_ready_o & rx_req_trac_valid_i) begin
        rx_req_valid <= 1'b1;
        rx_req_last  <= rx_req_trac_last_i;
    end
    else begin
        rx_req_valid <= 1'b0;
        rx_req_last  <= 1'b0;
    end
end

always @(posedge clk_i) begin
    if (rst_i) begin
        rx_req_start  <= 1'b1;
    end
    else if(rx_req_valid & rx_req_last) begin
        rx_req_start  <= 1'b1;
    end
    else if(rx_req_valid & rx_req_start) begin
        rx_req_start  <= 1'b0;
    end
end

always @(posedge clk_i) begin
    rx_req_data           <= rx_req_trac_data_i;
    rx_req_idinfo         <= rx_req_trac_idinfo_i;
    srio_trac_start_1d    <= srio_trac_start_i;
    srio_trac_start_2d    <= srio_trac_start_1d;
    srio_trac_start_3d    <= srio_trac_start_2d;
    srio_rx_req_raddr_1d  <= srio_rx_req_raddr_i;
    srio_rx_req_raddr_2d  <= srio_rx_req_raddr_1d;
    rx_req_dout_1d        <= rx_req_dout;
    rx_req_dout_2d        <= rx_req_dout_1d;
    fifo_nafull_1d        <= ~fifo_afull;
    fifo_nafull_2d        <= fifo_nafull_1d;
end

always @(posedge clk_i) begin
    if((srio_trac_start_2d == 1'b1) && (srio_trac_start_3d == 1'b0))
        rx_req_waddr   <= 9'd0;
    else if(rx_req_wen == 1'b1)
        rx_req_waddr   <= rx_req_waddr + 9'd1;
end

always @(posedge clk_i) begin
    rx_req_wen        <= rx_req_valid;
    rx_req_din        <= {rx_req_idinfo,rx_req_last,rx_req_data};
end

gowin_sdpram #
(
    .RAM_WIDTH         ( 64+32+1                ),                
    .RAM_DEPTH         ( 512                    ),                
    .RAM_PERFORMANCE   ( "bypass"               ),          
    .RAM_STYLE         ( "block_ram"            )
)                      
u0_sdpram_dat          
(                      
    .i_addra           ( rx_req_waddr           ),
    .i_addrb           ( srio_rx_req_raddr_2d   ),
    .i_dina            ( rx_req_din             ), 
    .i_clka            ( clk_i                  ),     
    .i_clkb            ( clk_i                  ), 
    .i_wea             ( rx_req_wen             ),  
    .i_enb             ( 1'b1                   ),  
    .i_rstb            ( 1'b0                   ), 
    .i_regceb          ( 1'b1                   ),
    .o_doutb           ( rx_req_dout            )
);

///reponse gen
always @(posedge clk_i) begin
    if (rst_i)
        header_valid <= 1'b0;
    else if((rx_req_start == 1'b1) & (rx_req_valid == 1'b1))
        header_valid        <= 1'b1;
    else
        header_valid        <= 1'b0;
end

always @(posedge clk_i) begin
    if((rx_req_start == 1'b1) & (rx_req_valid == 1'b1)) begin
        current_tid       <= rx_req_data[63:56];
        current_ftype     <= rx_req_data[55:52];
        current_ttype     <= rx_req_data[51:48];
        current_size      <= rx_req_data[43:36];
        current_prio      <= rx_req_data[46:45] + 2'b01;
        current_addr      <= rx_req_data[33:0];
        current_srcid     <= rx_req_data[31:16];
        current_hopcount  <= rx_req_data[31:24]; 
        current_idinfo    <= rx_req_idinfo;
    end
end

// Reply to response transaction
always @(posedge clk_i) begin
    if (rst_i) begin
        fifo_wen <= 1'b0;
    end 
    else if (header_valid == 1'b1) begin
        fifo_wen <= (current_ftype == NREAD) ||
                    (current_ftype == DOORB) ||
                    (current_ftype == MESSG) ||
                    ((current_ftype == NWRITE) && (current_ttype == TNWR_R));
    end 
    else begin
        fifo_wen <= 1'b0;
    end
end

always @(posedge clk_i) begin
    fifo_wdata <= {current_idinfo,5'h0,current_hopcount, current_ttype, current_srcid, current_addr[23:16] == 8'h12, current_addr[10:3],
                   current_prio, current_tid, current_ftype, current_size};
end

fifo_sc_top u_fifo_top
(
///	.WrReset       ( rst_i            ), //input WrReset
///	.RdReset       ( rst_i            ), //input RdReset
///	.WrClk         ( clk_i            ), //input WrClk
///	.RdClk         ( clk_i            ), //input RdClk
    .Clk           ( clk_i            ),
    .Reset         ( rst_i            ),
    
	.WrEn          ( fifo_wen         ), //input WrEn
	.Data          ( fifo_wdata       ), //input [95:0] Data
	.Almost_Full   ( fifo_afull       ), //output Almost_Full
	.Full          (                  ), //output Full
    
	.RdEn          ( fifo_ren         ), //input RdEn
	.Q             ( fifo_rdata       ), //output [95:0] Q
	.Almost_Empty  (                  ), //output Almost_Empty
	.Empty         ( fifo_empty       )  //output Empty

);

localparam                              S_IDLE    = 3'b000;
localparam                              S_INF     = 3'b001;
localparam                              S_RD      = 3'b010;
localparam                              S_WAIT    = 3'b011;
localparam                              S_HEAD    = 3'b100;
localparam                              S_TDATA   = 3'b101;

reg [2:0]                               state = S_IDLE/* synthesis preserve = 1 */;
reg [2:0]                               next_state/* synthesis preserve = 1 */;       

always@(posedge clk_i) begin
    if(rst_i)
        state <= S_IDLE;
    else 
        state <= next_state;
end

always@(*) begin
    case(state)
        S_IDLE : 
            begin
                if(fifo_empty == 1'b0)///((tx_resp_trac_ready_i == 1'b1) & (fifo_empty == 1'b0))
                    next_state = S_INF;
                else 
                    next_state = S_IDLE;
            end
        
        S_INF :
            begin
                next_state = S_RD;
            end
            
        S_RD :
            begin
                next_state = S_WAIT;
            end

        S_WAIT :
            begin
                next_state = S_HEAD;
            end

        S_HEAD :
            begin
                if((responding_ttype == TNDATA || responding_ttype == MSGRSP) && (tx_resp_trac_valid_o == 1'b1) && (tx_resp_trac_ready_i == 1'b1))
                    next_state = S_IDLE;
                else if((tx_resp_trac_valid_o == 1'b1) && (tx_resp_trac_ready_i == 1'b1))
                    next_state = S_TDATA;
                else 
                    next_state = S_HEAD;
            end

        S_TDATA :
            begin
                if((tx_resp_trac_valid_o == 1'b1) && (tx_resp_trac_ready_i == 1'b1) && (tx_resp_trac_last_o == 1'b1))
                    next_state = S_IDLE;
                else 
                    next_state = S_TDATA;
            end

        
        default :
            begin
                next_state = S_IDLE;
            end
    endcase
end

///assign fifo_ren                = ((state == S_IDLE) & (fifo_empty == 1'b0)) ? 1'b1 : 1'b0;///& (tx_resp_trac_ready_i == 1'b1) & (fifo_empty == 1'b0)) ? 1'b1 : 1'b0;
assign responding_ttype        = (response_ftype == NREAD) ? TWDATA : (response_ftype == MESSG) ? MSGRSP : TNDATA;
assign header_beat             = {response_tid, RESP, responding_ttype, 1'b0, response_prio, 45'h0};
assign tx_resp_trac_valid_o    = ((state == S_TDATA) | (state == S_HEAD)) ? 1'b1 : 1'b0;
assign tx_resp_trac_last_o     = tdata_last;
assign tx_resp_trac_data_o     = tdata_out;

always @(posedge clk_i) begin
    if (rst_i) begin
        tx_resp_state_end             <= 1'b0;
    end
    else if((state == S_HEAD) & (responding_ttype == TNDATA || responding_ttype == MSGRSP) && (tx_resp_trac_valid_o == 1'b1) && (tx_resp_trac_ready_i == 1'b1)) begin
        tx_resp_state_end             <= 1'b1;
    end
    else if((state == S_TDATA) & (tx_resp_trac_valid_o == 1'b1) && (tx_resp_trac_ready_i == 1'b1) && (tx_resp_trac_last_o == 1'b1)) begin
        tx_resp_state_end             <= 1'b1;
    end
    else begin
        tx_resp_state_end             <= 1'b0;
    end
end


always @(posedge clk_i) begin
    if((state == S_IDLE) & (fifo_empty == 1'b0)) begin
        fifo_ren          <= 1'b1;
    end
    else begin
        fifo_ren          <= 1'b0;
    end
end

always @(posedge clk_i) begin
    if(fifo_ren_1d == 1'b1) begin
        response_tid          <= fifo_rdata[19:12];
        response_ftype        <= fifo_rdata[11:8];
        response_ttype        <= fifo_rdata[50:47];
        response_size         <= fifo_rdata[7:0];
        response_prio         <= fifo_rdata[21:20];
        dest_id               <= fifo_rdata[46:31];
        starting_read_addr    <= {1'b0, fifo_rdata[29:22]};
        pull_from_store       <= fifo_rdata[30];
        response_hopcount     <= fifo_rdata[58:51] - 8'd1; 
        response_idinfo       <= {fifo_rdata[79:64],fifo_rdata[95:80]};
    end
end

always @(posedge clk_i) begin
    tx_resp_trac_idinfo_o     <= response_idinfo;
    fifo_ren_1d               <= fifo_ren;
end

always @(posedge clk_i) begin
    if(state == S_WAIT) begin
        tdata_size   <= response_size[7:3];
    end
    else if((state == S_TDATA) && (tx_resp_trac_ready_i == 1'b1) && (tx_resp_trac_valid_o == 1'b1) && (tdata_size > 5'd0)) begin
        tdata_size   <= tdata_size - 5'd1;
    end
end

always @(posedge clk_i) begin
    if(state == S_RD) begin
        tdata_start   <= 1'b1;
    end
    else if((tdata_start == 1'b1) && (tx_resp_trac_ready_i == 1'b1) && (tx_resp_trac_valid_o == 1'b1)) begin
        tdata_start   <= 1'b0;
    end
end

always @(posedge clk_i) begin
    if((state == S_WAIT) && (responding_ttype == TNDATA || responding_ttype == MSGRSP))begin
        tdata_last   <= 1'b1;
    end
    else if(state == S_WAIT) begin
        tdata_last   <= 1'b0;
    end
    else if((tdata_last == 1'b1) && (tx_resp_trac_ready_i == 1'b1) && (tx_resp_trac_valid_o == 1'b1)) begin
        tdata_last   <= 1'b0;
    end
    else if((state == S_TDATA) && (tx_resp_trac_ready_i == 1'b1) && (tx_resp_trac_valid_o == 1'b1) && (tdata_size == 5'd1)) begin
        tdata_last   <= 1'b1;
    end
    else if((state == S_HEAD) && (responding_ttype != TNDATA) && (responding_ttype != MSGRSP) && (tx_resp_trac_valid_o == 1'b1) && (tx_resp_trac_ready_i == 1'b1) && (tdata_size == 5'd0)) begin
        tdata_last   <= 1'b1;
    end
end

always @(posedge clk_i) begin
    if((tdata_start == 1'b1) && ((tx_resp_trac_ready_i == 1'b0) || (tx_resp_trac_valid_o == 1'b0))) begin
        tdata_out   <= header_beat;
    end
    else begin
        tdata_out   <= {8{increase_data}};
    end
end

always @* begin
    increase_data = increase_data_1d;
    if((tdata_start == 1'b0) && (tx_resp_trac_ready_i == 1'b1) && (tx_resp_trac_valid_o == 1'b1)) begin
        increase_data   = increase_data_1d + 8'h1;
    end
end

always @(posedge clk_i) begin
    if((srio_trac_start_2d == 1'b1) && (srio_trac_start_3d == 1'b0)) begin
        increase_data_1d   <= 8'h00;   
    end
    else begin
        increase_data_1d   <= increase_data;
    end
end

endmodule


