
`timescale 1 ns / 1 ps

`include "../../project/src/serdes/srio/temp/srio/srio_defines.vh"

module tc_srio;


///localparam								DEVICE_ID = 16'h0304;///8'h34;
localparam                              DEST_ID   = 16'hAFAF;///8'haf;
localparam                              TEST_MODE = 0;///0 single test, 1 continuous test

// clock & reset 
///wire                                    sys_reset_n;
reg                                     sys_reset_n;

reg                                     sys_clk = 0; 
reg[2:0]                                srio_rst_srl;
wire                                    sys_rst;
reg                                     cfg_clk = 0;
wire                                    cfg_rst;

///wire                                    gt_pcs_clk;
reg                                     gt_pcs_clk=0;
reg[2:0]                                gt_pcs_rst_srl;
reg[2:0]                                cfg_rst_srl;
wire                                    gt_pcs_rst;
wire                                    srio_ui_rst; 
wire                                    srio_ui_clk; 

initial begin
    sys_reset_n = 1'b0;
    #1005;
    sys_reset_n = 1'b1;
end

`ifdef LINE_RATE_1P25G
generate 
if(`LANE_WIDTH == 1) begin
    always #16  gt_pcs_clk  = ~gt_pcs_clk;   // @32bit
    always #32  sys_clk     = ~sys_clk;      // half gt_pcs_clk @64bit
end
else if(`LANE_WIDTH == 2) begin
    always #16  gt_pcs_clk  = ~gt_pcs_clk;   // @32bit*2
    always #16  sys_clk     = ~sys_clk;      // @64bit
end
else begin ///(`LANE_WIDTH == 4) 
    always #16  gt_pcs_clk  = ~gt_pcs_clk;   // @32bit*4
    always #8   sys_clk     = ~sys_clk;      // @64bit
end
endgenerate
`endif

`ifdef LINE_RATE_2P5G
generate 
if(`LANE_WIDTH == 1) begin
    always #8  gt_pcs_clk  = ~gt_pcs_clk;   // @32bit
    always #16  sys_clk     = ~sys_clk;      // half gt_pcs_clk @64bit
end
else if(`LANE_WIDTH == 2) begin
    always #8  gt_pcs_clk  = ~gt_pcs_clk;   // @32bit*2
    always #8  sys_clk     = ~sys_clk;      // @64bit
end
else begin ///(`LANE_WIDTH == 4) 
    always #8  gt_pcs_clk  = ~gt_pcs_clk;   // @32bit*4
    always #4   sys_clk     = ~sys_clk;      // @64bit
end
endgenerate
`endif

`ifdef LINE_RATE_3P125G
generate 
if(`LANE_WIDTH == 1) begin
    always #6.4  gt_pcs_clk  = ~gt_pcs_clk;   // @32bit
    always #12.8 sys_clk     = ~sys_clk;      // half gt_pcs_clk @64bit
end
else begin ///if (`LANE_WIDTH == 2)
    always #6.4  gt_pcs_clk  = ~gt_pcs_clk;   // @32bit*2
    always #6.4  sys_clk     = ~sys_clk;      // @64bit
end
endgenerate
`endif

`ifdef LINE_RATE_5G
generate 
if(`LANE_WIDTH == 1) begin
    always #4  gt_pcs_clk  = ~gt_pcs_clk;   // @32bit
    always #8  sys_clk     = ~sys_clk;      // half gt_pcs_clk @64bit
end
endgenerate
`endif

always@(posedge sys_clk or negedge sys_reset_n) begin
    if(!sys_reset_n)
        srio_rst_srl[2:0] <= 3'b111;
    else 
        srio_rst_srl[2:0] <= {srio_rst_srl[1:0],1'b0};
end

always@(posedge gt_pcs_clk or negedge sys_reset_n) begin
    if(!sys_reset_n)
        gt_pcs_rst_srl[2:0] <= 3'b111;
    else 
        gt_pcs_rst_srl[2:0] <= {gt_pcs_rst_srl[1:0],1'b0};
end

always@(posedge cfg_clk or negedge sys_reset_n) begin
    if(!sys_reset_n)
        cfg_rst_srl[2:0] <= 3'b111;
    else 
        cfg_rst_srl[2:0] <= {cfg_rst_srl[1:0],1'b0};
end

assign                                  sys_rst     = srio_rst_srl[2];
assign                                  gt_pcs_rst  = gt_pcs_rst_srl[2];

always #50                              cfg_clk     = ~cfg_clk; 
assign                                  cfg_rst     = cfg_rst_srl[2];


// user interface TX           
wire                                    tx_req_trac_valid;
wire                                    tx_req_trac_ready;
wire                                    tx_req_trac_last;
wire[63:0]                              tx_req_trac_data;
wire[2*`DEVICE_ID_W-1:0]                tx_req_trac_id_info;
wire[31:0]                              tx_req_trac_idinfo_temp;

wire                                    tx_resp_trac_valid;
wire                                    tx_resp_trac_ready;
wire                                    tx_resp_trac_last;
wire[63:0]                              tx_resp_trac_data;
wire[2*`DEVICE_ID_W-1:0]                tx_resp_trac_id_info;
wire[31:0]                              tx_resp_trac_idinfo_temp;

// user interface RX            
wire                                    rx_req_trac_ready;
wire                                    rx_req_trac_valid;
wire                                    rx_req_trac_last;
wire[63:0]                              rx_req_trac_data;
wire[2*`DEVICE_ID_W-1:0]                rx_req_trac_id_info;
wire[31:0]                              rx_req_trac_idinfo_temp;

wire                                    rx_resp_trac_ready;
wire                                    rx_resp_trac_valid;
wire                                    rx_resp_trac_last;
wire[63:0]                              rx_resp_trac_data;
wire[2*`DEVICE_ID_W-1:0]                rx_resp_trac_id_info;
wire[31:0]                              rx_resp_trac_idinfo_temp;

wire  [15:0]                            srio_device_id;
wire  [15:0]                            srio_dest_id;

wire                                    port_initialized;
wire                                    link_initialized;

wire[1:0]                               request_autocheck_error;
wire[1:0]                               request_done;

wire                                    cfg2_psel;
wire                                    cfg2_pwrite;
wire                                    cfg2_penable;
wire [21:0]                             cfg2_paddr;///byte address
wire [31:0]                             cfg2_pwdata;
wire                                    cfg2_pready;
wire [31:0]                             cfg2_prdata;

// Serdes interface
wire [`LANE_WIDTH-1:0]                  gtr12_tx_valid;
wire [80*`LANE_WIDTH-1:0]               gtr12_tx_dat;  
wire [`LANE_WIDTH-1:0]                  gtr12_rx_valid;    
wire [88*`LANE_WIDTH-1:0]               gtr12_rx_dat;

reg  [3:0]                              link_initialized_cnt=0;
reg  [10:0]                             maints_link_initialized_cnt=0;
reg                                     test_req;
wire                                    test_matins_req;
reg  [15:0]                             error_cnt=0;
reg  [15:0]                             done_cnt=0;
reg                                     request_done_1d;
reg                                     request_done_2d;
reg                                     request_err_1d; 
reg                                     maints_request_done_1d;

generate
if(`DEVICE_ID_W == 8) begin
    assign                              srio_dest_id               = {8'd0,DEST_ID[7:0]};
    assign                              tx_req_trac_id_info        = {tx_req_trac_idinfo_temp[23:16],tx_req_trac_idinfo_temp[7:0]};
    assign                              tx_resp_trac_id_info       = {tx_resp_trac_idinfo_temp[23:16],tx_resp_trac_idinfo_temp[7:0]};   
    assign                              rx_resp_trac_idinfo_temp   = {8'b0,rx_resp_trac_id_info[15:8],8'b0,rx_resp_trac_id_info[7:0]};
    assign                              rx_req_trac_idinfo_temp    = {8'b0,rx_req_trac_id_info[15:8],8'b0,rx_req_trac_id_info[7:0]};
end                                     
else begin                              
    assign                              srio_dest_id               = DEST_ID[15:0];
    assign                              tx_req_trac_id_info        = {tx_req_trac_idinfo_temp[31:16],tx_req_trac_idinfo_temp[15:0]};
    assign                              tx_resp_trac_id_info       = {tx_resp_trac_idinfo_temp[31:16],tx_resp_trac_idinfo_temp[15:0]};   
    assign                              rx_resp_trac_idinfo_temp   = {rx_resp_trac_id_info[31:16],rx_resp_trac_id_info[15:0]};
    assign                              rx_req_trac_idinfo_temp    = {rx_req_trac_id_info[31:16],rx_req_trac_id_info[15:0]};
end
endgenerate

genvar i;
generate
    for (i=0; i<`LANE_WIDTH; i=i+1) begin : group0 
        assign          gtr12_rx_dat[8+88*i]          = gtr12_tx_dat[08+80*i];
        assign          gtr12_rx_dat[18+88*i]         = gtr12_tx_dat[18+80*i];
        assign          gtr12_rx_dat[28+88*i]         = gtr12_tx_dat[28+80*i];
        assign          gtr12_rx_dat[38+88*i]         = gtr12_tx_dat[38+80*i];
        assign          gtr12_rx_dat[7+88*i:0+88*i]   = gtr12_tx_dat[7+80*i:0+80*i];
        assign          gtr12_rx_dat[17+88*i:10+88*i] = gtr12_tx_dat[17+80*i:10+80*i];
        assign          gtr12_rx_dat[27+88*i:20+88*i] = gtr12_tx_dat[27+80*i:20+80*i];
        assign          gtr12_rx_dat[37+88*i:30+88*i] = gtr12_tx_dat[37+80*i:30+80*i];
        assign          gtr12_rx_dat[9+88*i]          = 1'b0;
        assign          gtr12_rx_dat[19+88*i]         = 1'b0;
        assign          gtr12_rx_dat[29+88*i]         = 1'b0;
        assign          gtr12_rx_dat[87+88*i:39+88*i] = 49'b0;
    end
endgenerate

assign                  gtr12_rx_valid                = gtr12_tx_valid;
assign                  test_matins_req               = maints_link_initialized_cnt[10];

always @(posedge srio_ui_clk) begin
    if (srio_ui_rst) begin
        maints_link_initialized_cnt <= 0;
    end 
    else if (link_initialized && !test_matins_req) begin
        maints_link_initialized_cnt <= maints_link_initialized_cnt + 1'b1;
    end 
    else if (!link_initialized) begin
        maints_link_initialized_cnt <= 0;
    end
end

always @(posedge srio_ui_clk) begin
    if (srio_ui_rst) begin
        link_initialized_cnt <= 0;
    end 
    else if (link_initialized && !test_req && maints_request_done_1d) begin
        link_initialized_cnt <= link_initialized_cnt + 1'b1;
    end 
    else if (!link_initialized) begin
        link_initialized_cnt <= 0;
    end
end

always @(posedge srio_ui_clk) begin
    if (srio_ui_rst) begin
        test_req <= 0;
    end 
    else if (link_initialized && request_done[0] && (TEST_MODE == 1)) begin
        test_req <= 1'b0;
    end 
    else if (link_initialized_cnt[3]) begin
        test_req <= 1'b1;
    end
end

always @(posedge srio_ui_clk) begin
    request_done_1d        <= request_done[0];
    request_done_2d        <= request_done_1d;
    request_err_1d         <= request_autocheck_error[0];
    maints_request_done_1d <= request_done[1];
end

always @(posedge srio_ui_clk) begin
    if (request_done_1d && (!request_done_2d) && (request_err_1d)) begin
        error_cnt <= error_cnt + 1'b1;
    end 
end

always @(posedge srio_ui_clk) begin
    if (request_done_1d && (!request_done_2d)) begin
        done_cnt <= done_cnt + 1'b1;
    end 
end

//----------------------------------------
//--------------------  RapidIO IP 
//----------------------------------------
SRIO_Top u_SRIO_Top
(

    .sys_rst_i                         ( sys_rst                       ),    
    .srio_clk_i                        ( sys_clk                       ),
    .clk_lock_i                        ( 1'b1                          ),
    .user_pcs_clk_i                    ( gt_pcs_clk                    ),
                                       
    .srio_ui_clk_o                     ( srio_ui_clk                   ),
    .srio_ui_rst_o                     ( srio_ui_rst                   ),
                                       
    // --------- user interface TX     
    .tx_req_trac_valid_i               ( tx_req_trac_valid             ),
    .tx_req_trac_ready_o               ( tx_req_trac_ready             ),
    .tx_req_trac_last_i                ( tx_req_trac_last              ),
    .tx_req_trac_data_i                ( tx_req_trac_data              ),
    .tx_req_trac_id_info_i             ( tx_req_trac_id_info           ),
                                       
    .tx_resp_trac_valid_i              ( tx_resp_trac_valid            ),
    .tx_resp_trac_ready_o              ( tx_resp_trac_ready            ),
    .tx_resp_trac_last_i               ( tx_resp_trac_last             ),
    .tx_resp_trac_data_i               ( tx_resp_trac_data             ),
    .tx_resp_trac_id_info_i            ( tx_resp_trac_id_info          ),

    // --------- user interface RX     
    .rx_req_trac_ready_i               ( rx_req_trac_ready             ),
    .rx_req_trac_valid_o               ( rx_req_trac_valid             ),
    .rx_req_trac_last_o                ( rx_req_trac_last              ),
    .rx_req_trac_data_o                ( rx_req_trac_data              ),
    .rx_req_trac_id_info_o             ( rx_req_trac_id_info           ),

    .rx_resp_trac_ready_i              ( rx_resp_trac_ready            ),
    .rx_resp_trac_valid_o              ( rx_resp_trac_valid            ),
    .rx_resp_trac_last_o               ( rx_resp_trac_last             ),
    .rx_resp_trac_data_o               ( rx_resp_trac_data             ),
    .rx_resp_trac_id_info_o            ( rx_resp_trac_id_info          ),

    // --------- PHY control signals  
    .force_reinit_i                    ( 1'b0                          ),
    .phy_tx_mce_i                      ( 1'b0                          ),
    .phy_tx_link_reset_i               ( 1'b0                          ),
                                                                       
    //-------- PHY Informational signals                               
    .port_initialized_o                ( port_initialized              ),
    .link_initialized_o                ( link_initialized              ),
                                                                       
    // --------- core debug signals                                    
    .phy_rx_mce_o                      (                               ),
    .phy_rx_link_reset_o               (                               ),
    .port_error_o                      (                               ),
    .port_timeout_o                    (                               ),
    .srio_host_o                       (                               ),
    .master_enable_o                   (                               ),
    .port_decode_error_o               (                               ),
    .device_id_o                       ( srio_device_id                ),
    .debug_info_o                      (                               ),
    .rxbuf_rewind_o                    (                               ),
    .txbuf_rewind_o                    (                               ),
                                                                       
    // --------- Maintenance Bus                                       
    .cfg_clk_i                         ( cfg_clk                       ),
    .cfg_rst_i                         ( cfg_rst                       ),
    .cfg_sel_i                         ( cfg2_psel                     ),///cfg1_psel                 ),
    .cfg_wr_i                          ( cfg2_pwrite                   ),///cfg1_pwrite               ),
    .cfg_ena_i                         ( cfg2_penable                  ),///cfg1_penable              ),
    .cfg_addr_i                        ( cfg2_paddr                    ),///cfg1_paddr                ),///byte address
    .cfg_wdata_i                       ( cfg2_pwdata                   ),///cfg1_pwdata               ),
    .cfg_strb_i                        ( 4'hf                          ),
    .cfg_rdy_o                         ( cfg2_pready                   ),///cfg1_pready               ),
    .cfg_rdata_o                       ( cfg2_prdata                   ),///cfg1_prdata               ),
    .cfg_slverr_o                      (                               ),
                                                                       
    .maintenance_interrupt_o           (                               ),
    .maintenance_timeout_o             (                               ),

    //-------- User define serdes control
    .gtr12_rstn_i                      ( 1'b1                          ),
    .gtr12_pcs_tx_rst_i                ( 1'b0                          ),
    .gtr12_pcs_rx_rst_i                ( 1'b0                          ),
    .gtr12_align_link_o                (                               ),
    .gtr12_k_lock_o                    (                               ),
    .gtr12_pma_rx_lock_o               (                               ),
    .gtr12_pll_ok_o                    (                               ),
    .gtr12_pll_ck_ref_o                (                               ),
    .gtr12_rx_fifo_rdusewd_o           (                               ),
    .gtr12_tx_fifo_wrusewd_o           (                               ),
    .gtr12_tx_pcs_clk_o                (                               ),
    .gtr12_rx_pcs_clk_o                (                               ),

    //-------- Serdes interface                                        
    //This set of ports connects to the Serdes Hard Core. Gowin EDA software will automatically connect.
    //For simulation only
    .serdes_lnx_rstn_o                 (                               ),///FABRIC_LN#_RSTN_I
    .serdes_pcs_tx_rst_o               (                               ),///LANE#_PCS_TX_RST
    .serdes_pcs_rx_rst_o               (                               ),///LANE#_PCS_RX_RST
    .serdes_lanex_align_link_i         ( {`LANE_WIDTH{1'b1}}           ),///LANE#_ALIGN_LINK
    .serdes_lanex_k_lock_i             ( {`LANE_WIDTH{1'b1}}           ),///LANE#_K_LOCK
    .serdes_lnx_pma_rx_lock_i          ( {`LANE_WIDTH{1'b1}}           ),///FABRIC_LN#_PMA_RX_LOCK_O
    .serdes_cmu1_ok_i                  ( 1'b1                          ),///FABRIC_CMU1_OK_O
    .serdes_cmu1_ck_ref_i              ( 1'b0                          ),///FABRIC_CMU1_CK_REF_O
    .serdes_cmu0_ok_i                  ( 1'b1                          ),///FABRIC_CMU_OK_O
    .serdes_cmu0_ck_ref_i              ( 1'b0                          ),///FABRIC_CMU_CK_REF_O
    .serdes_lanex_cmu_ok_i             ( {`LANE_WIDTH{1'b1}}           ),///FABRIC_LANE#_CMU_OK_O
    .serdes_lanex_cmu_ck_ref_i         ( {`LANE_WIDTH{1'b0}}           ),///FABRIC_LANE#_CMU_CK_REF_O
    .serdes_lanex_pcs_rx_fabric_clk_i  ( {`LANE_WIDTH{gt_pcs_clk}}     ),///LANE#_PCS_RX_O_FABRIC_CLK
    .serdes_lanex_pcs_tx_fabric_clk_i  ( {`LANE_WIDTH{gt_pcs_clk}}     ),///LANE#_PCS_TX_O_FABRIC_CLK
    .serdes_lanex_chbond_start_o       (                               ),///LANE#_CHBOND_START       
    .serdes_lanex_align_trigger_o      (                               ),///LANE#_ALIGN_TRIGGER
    .serdes_lanex_fabric_rx_clk_o      (                               ),///LANE#_FABRIC_RX_CLK
    .serdes_lanex_fabric_c2i_clk_o     (                               ),///LANE#_FABRIC_C2I_CLK
    .serdes_lanex_fabric_tx_clk_o      (                               ),///LANE#_FABRIC_TX_CLK
    .serdes_lanex_rx_if_fifo_aempty_i  ( {`LANE_WIDTH{1'b0}}           ),///LANE#_RX_IF_FIFO_AEMPTY
    .serdes_lanex_tx_if_fifo_full_i    ( {`LANE_WIDTH{1'b0}}           ),///LANE#_TX_IF_FIFO_FULL
    .serdes_lanex_rx_if_fifo_rden_o    (                               ),///LANE#_RX_IF_FIFO_RDEN
    .serdes_lnx_tx_vld_o               ( gtr12_tx_valid                ),///FABRIC_LN#_TX_VLD_IN
    .serdes_lnx_rx_vld_i               ( gtr12_rx_valid                ),///FABRIC_LN#_RX_VLD_OUT
    .serdes_lnx_rxdata_i               ( gtr12_rx_dat                  ),///FABRIC_LN#_RXDATA_O
    .serdes_lnx_txdata_o               ( gtr12_tx_dat                  ),///FABRIC_LN#_TXDATA_I  
    .serdes_rx_if_fifo_rdusewd_i       ( {`LANE_WIDTH{5'b0}}           ),///LANE#_RX_IF_FIFO_RDUSEWD   
    .serdes_tx_if_fifo_wrusewd_i       ( {`LANE_WIDTH{5'b0}}           ) ///LANE#_TX_IF_FIFO_WRUSEWD
    
);

gowin_srio_maints u_gowin_srio_maints 
(
    .clk_i                             ( cfg_clk                          ),
    .rst_i                             ( cfg_rst                          ),
                                                                         
    .cfg_sel_o                         ( cfg2_psel                        ),
    .cfg_wr_o                          ( cfg2_pwrite                      ),
    .cfg_ena_o                         ( cfg2_penable                     ),
    .cfg_addr_o                        ( cfg2_paddr                       ),
    .cfg_wdata_o                       ( cfg2_pwdata                      ),
    .cfg_rdy_i                         ( cfg2_pready                      ),
    .cfg_rdata_i                       ( cfg2_prdata                      ),
                                      
    .srio_trac_start_i                 ( test_matins_req                  ),
    .maint_done_o                      ( request_done[1]                  ),
    .maint_autocheck_error_o           ( request_autocheck_error[1]       )
);

//Send request transactions, receive response transactions
gowin_srio_request_gen u0_srio_request_gen 
(
    .clk_i                             ( srio_ui_clk                      ),
    .rst_i                             ( srio_ui_rst                      ),
                                                                         
    .deviceid_i                        ( srio_device_id                   ),
    .dest_id_i                         ( srio_dest_id                     ),
                                       
    .tx_req_tvalid_o                   ( tx_req_trac_valid                ),
    .tx_req_tready_i                   ( tx_req_trac_ready                ),
    .tx_req_tlast_o                    ( tx_req_trac_last                 ),
    .tx_req_tdata_o                    ( tx_req_trac_data                 ),
    .tx_req_tidinfo_o                  ( tx_req_trac_idinfo_temp          ),
                                                                       
    .rx_resp_tvalid_i                  ( rx_resp_trac_valid               ),
    .rx_resp_tready_o                  ( rx_resp_trac_ready               ),
    .rx_resp_tlast_i                   ( rx_resp_trac_last                ),
    .rx_resp_tdata_i                   ( rx_resp_trac_data                ),
    .rx_resp_tidinfo_i                 ( rx_resp_trac_idinfo_temp         ),
                                       
    .srio_trac_start_i                 ( test_req                         ),
                                       
    .request_autocheck_error_o         ( request_autocheck_error[0]       ),
    .request_done_o                    ( request_done[0]                  )
);


//Receive request transactions and send back response transactions
gowin_srio_response_gen u0_srio_response_gen 
(
    .clk_i                             ( srio_ui_clk                      ),
    .rst_i                             ( srio_ui_rst                      ),
                                                                          
    .deviceid_i                        ( srio_device_id                   ),
    .source_id_i                       ( 16'b0                            ),
                                                                          
    .srio_trac_start_i                 ( test_req                         ),
                                                                          
    .srio_rx_req_raddr_i               ( 9'd0                             ),
    .srio_rx_req_rd_data_o             (                                  ),
    .srio_rx_req_rd_last_o             (                                  ),
    .srio_rx_req_rd_idinfo_o           (                                  ),
                                                                          
    .tx_resp_trac_valid_o              ( tx_resp_trac_valid               ),
    .tx_resp_trac_ready_i              ( tx_resp_trac_ready               ),
    .tx_resp_trac_last_o               ( tx_resp_trac_last                ),
    .tx_resp_trac_data_o               ( tx_resp_trac_data                ),
    .tx_resp_trac_idinfo_o             ( tx_resp_trac_idinfo_temp         ),
                                                                          
    .rx_req_trac_valid_i               ( rx_req_trac_valid                ),
    .rx_req_trac_ready_o               ( rx_req_trac_ready                ),
    .rx_req_trac_last_i                ( rx_req_trac_last                 ),
    .rx_req_trac_data_i                ( rx_req_trac_data                 ),
    .rx_req_trac_idinfo_i              ( rx_req_trac_idinfo_temp          )
);


integer w_tx_req_file;
initial w_tx_req_file = $fopen("./tx_req_data.txt");
integer w_rx_req_file;
initial w_rx_req_file = $fopen("./rx_req_data.txt");
integer w_tx_resp_file;
initial w_tx_resp_file = $fopen("./tx_resp_data.txt");
integer w_rx_resp_file;
initial w_rx_resp_file = $fopen("./rx_resp_data.txt");

always@(posedge srio_ui_clk) begin
    if(tx_req_trac_valid & tx_req_trac_ready) 
        $fdisplay(w_tx_req_file,"%h",$unsigned(tx_req_trac_data));
    else if(request_done[0])
        $fclose(w_tx_req_file);
end

always@(posedge srio_ui_clk) begin
    if(rx_req_trac_valid & rx_req_trac_ready) 
        $fdisplay(w_rx_req_file,"%h",$unsigned(rx_req_trac_data));
    else if(request_done[0])
        $fclose(w_rx_req_file);
end

always@(posedge srio_ui_clk) begin
    if(tx_resp_trac_valid & tx_resp_trac_ready) 
        $fdisplay(w_tx_resp_file,"%h",$unsigned(tx_resp_trac_data));
    else if(request_done[0])
        $fclose(w_tx_resp_file);
end

always@(posedge srio_ui_clk) begin
    if(rx_resp_trac_valid & rx_resp_trac_ready) 
        $fdisplay(w_rx_resp_file,"%h",$unsigned(rx_resp_trac_data));
    else if(request_done[0])
        $fclose(w_rx_resp_file);
end

initial begin
  $fsdbDumpfile("tb.fsdb");
  $fsdbDumpvars;
end
  
initial begin 
	#7000_0000;
	$finish;
end

endmodule 
